﻿Imports System
Imports System.DirectoryServices
Imports Microsoft.VisualBasic
Imports System.Object
Imports System.Net.Mail
Imports System.Security.AccessControl
Imports System.Security.Principal
Imports CDOEXM
Imports System.Runtime.InteropServices

Public Class groupClass

    '-----------------------------------------------------------------------------------------------------------------------------------------
    '---------------------------------------------------------Generales-----------------------------------------------------------------------
    Public Shared Function GroupExists(ByVal GroupName As String) As Boolean
        '==========================================================================
        ' Name:     GroupExists
        ' Purpose:  Determinar si un grupo existe en el directorio activo 
        ' Input:    1 String: El nombre del grupo 'GroupName'
        ' Return:   1 Boolean: true si existe, false en caso contrario 
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del grupo
        Dim de As New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher()
        deSearch.SearchRoot = de
        deSearch.Filter = "(&(objectClass=group) (cn=" & GroupName & "))"
        Try
            Dim results As SearchResultCollection = deSearch.FindAll()

            'Si se ha obtenido devuelve true, sino false
            If results.Count = 0 Then
                Return False
            Else
                Return True
            End If
        Catch e As FormatException
            MsgBox("Error al buscar un grupo, Error de formato al buscar en el directorio activo al grupo  " & GroupName & e.Message)
        Catch e As AccessViolationException
            MsgBox("Error al buscar un grupo, No tienes permisos para buscar en el directorio activo al grupo " & GroupName & e.Message)
        Catch e As ArgumentOutOfRangeException
            MsgBox("Error al buscar un grupo, Se ha intentado obtener un dato con un índice fuera de rango al buscar en el directorio activo al grupo " & GroupName & e.Message)
        Catch e As OperationCanceledException
            MsgBox("Error al buscar un grupo, Se está intentando obtener un dato involucrado en una operación cancelada al buscar en el directorio activo al grupo " & GroupName & e.Message)
        Catch e As OperationAbortedException
            MsgBox("Error al buscar un grupo, Se está intentando obtener un dato involucrado en una operación abortada al buscar en el directorio activo al grupo " & GroupName & e.Message)
        Catch e As OutOfMemoryException
            MsgBox("Error al buscar un grupo, La memoria está totalmente ocupada al buscar en el directorio activo al grupo " & GroupName & e.Message)
        Catch e As FieldAccessException
            MsgBox("Error al buscar un grupo, Error al acceder a al ubicación física del dato al buscar en el directorio activo al grupo " & GroupName & e.Message)
        Catch e As InsufficientMemoryException
            MsgBox("Error al buscar un grupo, La memoria está totalmente ocupada al buscar en el directorio activo al grupo " & GroupName & e.Message)
        Catch e As MissingFieldException
            MsgBox("Error al buscar un grupo, Error al acceder a al ubicación física del dato, no existe al buscar en el directorio activo al grupo " & GroupName & e.Message)
        End Try
    End Function

    Public Shared Function GetDirectoryEntry() As DirectoryEntry
        '==========================================================================
        ' Name:     GetDirectoryEntry
        ' Purpose:  Obtener el DirectoryEntry de la raiz del servidor
        ' Input:    None
        ' Return:   1 DirectoryEntry: De la raiz del servidor
        ' Calls:    None
        '==========================================================================

        Try
            'Obtener el directoryEntry, darle permisos de controlador de dominio y devolverlo
            Dim de As DirectoryEntry = New DirectoryEntry()
            de.Username = Settings.adminLogon
            de.Password = Settings.adminPassword
            de.AuthenticationType = AuthenticationTypes.Delegation
            de.Path = "LDAP://" & Settings.ldapServerName & "/" & Settings.ADRoot
            de.Username = Settings.adminLogon
            de.Password = Settings.adminPassword
            de.AuthenticationType = AuthenticationTypes.Delegation
            Return de
        Catch e As FormatException
            MsgBox("Error al acceder al directorio activo, Error de formato del directorio activo  " & e.Message)
        Catch e As AccessViolationException
            MsgBox("Error al acceder al directorio activo, No tienes permisos para acceder al directorio activo " & e.Message)
        Catch e As ArgumentOutOfRangeException
            MsgBox("Error al acceder al directorio activo, Se ha intentado obtener un dato con un índice fuera de rango " & e.Message)
        Catch e As OperationCanceledException
            MsgBox("Error al acceder al directorio activo, Se está intentando obtener un dato involucrado en una operación cancelada " & e.Message)
        Catch e As OperationAbortedException
            MsgBox("Error al acceder al directorio activo, Se está intentando obtener un dato involucrado en una operación abortada " & e.Message)
        Catch e As OutOfMemoryException
            MsgBox("Error al acceder al directorio activo, La memoria está totalmente ocupada " & e.Message)
        Catch e As FieldAccessException
            MsgBox("Error al acceder al directorio activo, Error al acceder a al ubicación física del dato " & e.Message)
        Catch e As InsufficientMemoryException
            MsgBox("Error al acceder al directorio activo, La memoria está totalmente ocupada " & e.Message)
        Catch e As MissingFieldException
            MsgBox("Error al acceder al directorio activo, Error al acceder a al ubicación física del dato, no existe " & e.Message)
        End Try
    End Function

    Public Shared Sub CreateGroup(ByVal sGroupName As String, ByVal GroupDomain As String, ByVal GroupType As String, ByVal OU_AC As String)
        '==========================================================================
        ' Name:     CreateGroup
        ' Purpose:  Crear un nuevo Grupo
        ' Input:    4 string: El nombre del grupo 'sGroupName', el dominio al que pertenecerá 'GroupDomain',
        '           Tipo de grupo 'GroupType', Dirección en la que se creará 'OU_AC'
        ' Return:   None
        ' Calls:    None
        '==========================================================================

        Try
            'Constantes para los tipos de grupo
            Const Globalgroup As Integer = 2
            Const Domainlocalgroup As Integer = 4
            Const Universalgroup As Integer = 8
            Const SecurityUniversalgroup As Integer = -2147483640
            Const SecurityGlobalgroup As Integer = -2147483646
            Const SecurityLocalgroup As Integer = -2147483644

            'Obtener el container en el que se quiere crear el grupo
            Dim parts() As String = OU_AC.Split(",")
            ' Establecer el container como el lugar donde crear el nuevo grupo
            Dim dirE As New DirectoryEntry()
            Dim de As DirectoryEntry = dirE.Children.Find(parts(0))
            de.Username = Settings.adminLogon
            de.Password = Settings.adminPassword
            de.AuthenticationType = AuthenticationTypes.Delegation
            ' Crear el nuevo grupo
            Dim newGroup As DirectoryEntry = de.Children.Add("CN=" & sGroupName, "group")

            ' Asignarle al nuevo grupo el nombre y su tipo
            newGroup.Properties("sAMAccountName").Value = sGroupName
            If GroupType = "Security" Then
                If GroupDomain = "Local" Then
                    newGroup.Properties("groupType").Value = SecurityLocalgroup
                ElseIf GroupDomain = "Global" Then
                    newGroup.Properties("groupType").Value = SecurityGlobalgroup
                Else
                    newGroup.Properties("groupType").Value = SecurityUniversalgroup
                End If
            ElseIf GroupDomain = "Global" Then
                newGroup.Properties("groupType").Value = Globalgroup
            ElseIf GroupDomain = "Local" Then
                newGroup.Properties("groupType").Value = Domainlocalgroup
            Else
                newGroup.Properties("groupType").Value = Universalgroup
            End If

            newGroup.Properties("displayName").Add(sGroupName)
            ' Guardar el grupo
            Try
                newGroup.CommitChanges()
            Catch e As AccessViolationException
                MsgBox("Error al crear el nuevo grupo, no tiene permisos para crear nuevos grupos " & e.Message)
            Catch e As FormatException
                MsgBox("Error al crear el nuevo grupo, uno o varios de los atributos no tiene un formato válido " & e.Message)
            Catch e As ArgumentNullException
                MsgBox("Error al crear el nuevo grupo, algún atributo es nulo " & e.Message)
            End Try
        Catch COMEx As Exception
            'Devolver la excepción que será de tipo COM
            MsgBox(COMEx)
        End Try
    End Sub

    Public Shared Function getGroupadress(ByVal GroupName As String) As String
        '==========================================================================
        ' Name:     getGroupadress
        ' Purpose:  Devolver el atributo canonicalName del grupo
        ' Input:    1 string: El nombre del grupo 'GroupName'
        ' Return:   1 string: el atributo canonicalName es de tipo string
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del usuario
        Dim de As DirectoryEntry = GetDirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher()
        deSearch.SearchRoot = de
        deSearch.Filter = "(&(objectClass=group) (cn=" & GroupName & "))"
        Dim result As SearchResult = deSearch.FindOne()
        If result Is Nothing Then
            Return "no existe el grupo"
        Else
            Try
                'Devolver el atributo canonicalName si existe el grupo
                Dim group As New DirectoryEntry(result.Path)
                group.RefreshCache(New String() {"canonicalName"})
                Dim adress As String = group.Properties("canonicalName").Value.ToString
                Return adress
            Catch e As FormatException
                MsgBox("Error al acceder al nombre canónico del grupo, Error de formato al buscar en el directorio activo la dirección del grupo  " & GroupName & e.Message)
            Catch e As AccessViolationException
                MsgBox("Error al acceder al nombre canónico del grupo, No tienes permisos para buscar en el directorio activo la dirección del grupo " & GroupName & e.Message)
            Catch e As ArgumentOutOfRangeException
                MsgBox("Error al acceder al nombre canónico del grupo, Se ha intentado obtener un dato con un índice fuera de rango al buscar en el directorio activo la dirección del grupo " & GroupName & e.Message)
            Catch e As OperationCanceledException
                MsgBox("Error al acceder al nombre canónico del grupo, Se está intentando obtener un dato involucrado en una operación cancelada al buscar en el directorio activo la dirección del grupo " & GroupName & e.Message)
            Catch e As OperationAbortedException
                MsgBox("Error al acceder al nombre canónico del grupo, Se está intentando obtener un dato involucrado en una operación abortada al buscar en el directorio activo la dirección del grupo " & GroupName & e.Message)
            Catch e As OutOfMemoryException
                MsgBox("Error al acceder al nombre canónico del grupo, La memoria está totalmente ocupada al buscar en el directorio activo la dirección del grupo " & GroupName & e.Message)
            Catch e As FieldAccessException
                MsgBox("Error al acceder al nombre canónico del grupo, Error al acceder a al ubicación física del dato al buscar en el directorio activo la dirección del grupo " & GroupName & e.Message)
            Catch e As InsufficientMemoryException
                MsgBox("Error al acceder al nombre canónico del grupo, La memoria está totalmente ocupada al buscar en el directorio activo la dirección del grupo " & GroupName & e.Message)
            Catch e As MissingFieldException
                MsgBox("Error al acceder al nombre canónico del grupo, Error al acceder a al ubicación física del dato, no existe al buscar en el directorio activo la dirección del grupo " & GroupName & e.Message)
            End Try
        End If
    End Function

    Public Shared Sub CopyGroup(ByVal Grupo As String, ByVal Actual_OU As String)
        '==========================================================================
        ' Name:     CopyGroup
        ' Purpose:  Se realiza una copia del grupo
        ' Input:    2 string: El nombre del grupo 'GroupName', la unidad organizativa en la que se copia 'Actual_OU'
        ' Return:   1 string: el atributo canonicalName es de tipo string
        ' Calls:    None
        '==========================================================================
        Const Globalgroup As Integer = 2
        Const Domainlocalgroup As Integer = 4
        Const Universalgroup As Integer = 8
        Const SecurityUniversalgroup As Integer = -2147483640
        Const SecurityGlobalgroup As Integer = -2147483646
        Const SecurityLocalgroup As Integer = -2147483644

        Dim salida As String
        salida = "-------Copiar grupo-------"
        Dim Nnombre As String = "Copia de " & Grupo
        If Nnombre.Length > 20 Then
            MsgBox("No se puede copiar el grupo porque su nombre de Login no puede pasar de once caracteres")
        Else
            If Not GroupExists("Copia de " & Grupo) Then


                Dim typo As String = groupClass.GetGroupProperty(Grupo, "groupType")
                'Establecer el tipo de grupo y marcar los radiobuttons con su nombre deshabilitando los demás
                If typo = SecurityLocalgroup Then
                    CreateGroup(Nnombre, "Local", "Security", Actual_OU)
                ElseIf typo = Domainlocalgroup Then
                    CreateGroup(Nnombre, "Local", "Distribution", Actual_OU)
                ElseIf typo = SecurityGlobalgroup Then
                    CreateGroup(Nnombre, "Global", "Security", Actual_OU)
                ElseIf typo = Globalgroup Then
                    CreateGroup(Nnombre, "Global", "Distribution", Actual_OU)
                ElseIf typo = SecurityUniversalgroup Then
                    CreateGroup(Nnombre, "Universal", "Security", Actual_OU)
                ElseIf typo = Universalgroup Then
                    CreateGroup(Nnombre, "Usniversal", "Distribution", Actual_OU)
                End If
           

        '---------------------------General----------------------------------------

        Try
            Dim descrip As String = groupClass.GetGroupProperty(Grupo, "description")
            If Not descrip = "" Then
                groupClass.SetGroupProperty(Nnombre, "description", descrip)
            End If

        Catch ex As Exception
            MsgBox("Error al asignar la descripción al grupo " & ex.Message)
            salida = salida & vbCrLf & "Error: al asignar la descripción al grupo " & ex.Message

        End Try

        '---------------------------------------------Members----------------------------------------------------------------------------
        '-----------------------------------------------------------------------------------------------------------------------------------

        'MembersListView.Items.Clear()
        If Not groupClass.GetGroupPropertyObject(Grupo, "member") Is Nothing Then
            'Si hay mas de uno
            If Not groupClass.GetGroupPropertyObject(Grupo, "member").GetType = GetType(String) Then
                Dim Userslist() As Object = groupClass.GetGroupPropertyObject(Grupo, "member")
                'insertar los miembros en el listview
                For i = 0 To Userslist.Count - 1
                    Try
                        groupClass.SetGroupPropertyInsert(Nnombre, "member", Userslist(i).ToString, i)
                    Catch ex As Exception
                        MsgBox("Error al introducir el miembro " & Userslist(i).ToString & " al grupo " & ex.Message)
                        salida = salida & vbCrLf & "Error: al introducir el miembro " & Userslist(i).ToString & " al grupo " & ex.Message

                    End Try
                Next
            Else
                Try
                    groupClass.SetGroupProperty(Nnombre, "member", groupClass.GetGroupPropertyObject(Grupo, "member"))
                Catch ex As Exception
                    MsgBox("Error al introducir el miembro al grupo " & ex.Message)
                    salida = salida & vbCrLf & "Error: al introducir el miembro al grupo " & ex.Message

                End Try
            End If
        End If

        '---------------------------------------------Members of-----------------------------------------------------------------------
        '-----------------------------------------------------------------------------------------------------------------------------------
        Try
            salida = salida & vbCrLf & "cambios en los grupos de los que es miembro el grupo"

            'Obtener los demás grupos del usuario
            Dim grupos As List(Of String) = groupClass.GetGroupGroups(Grupo)

            If grupos.Count > 0 Then
                For i = 0 To grupos.Count - 1

                    'If Not grupos(i) = Prim Then
                    Dim DN As String = GenericClass.GetDN(grupos(i))

                    Dim logN As String = grupos(i)
                    Dim lnewlist As New List(Of String)
                    lnewlist = groupClass.GetGroupMembers(logN)
                    Try
                        groupClass.SetGroupPropertyInsert(logN, "member", GenericClass.GetDN(Nnombre), lnewlist.Count)
                    Catch ex As AccessViolationException
                        MsgBox("Error, por falta de permisos, al añadir el usuario al grupo " & logN & " " & ex.Message)
                        salida = salida & vbCrLf & "Error, por falta de permisos, al añadir el usuario al grupo " & logN & " " & ex.Message

                    Catch ex As FieldAccessException
                        MsgBox("Error, por falta de permisos, al añadir el usuario al grupo " & logN & " " & ex.Message)
                        salida = salida & vbCrLf & "Error, por falta de permisos, al añadir el usuario al grupo " & logN & " " & ex.Message

                    Catch ex As MemberAccessException
                        MsgBox("Error, por falta de permisos, al eliminar los grupos de los que es miembro el usuario para posteriormente insertar los nuevos" & ex.Message)
                        salida = salida & vbCrLf & "Error por falta de permisos: al eliminar los grupos de los que es miembro el equipo para posteriormente insertar los nuevos" & ex.Message

                    Catch ex As UnauthorizedAccessException
                        MsgBox("Error, por falta de permisos, al eliminar los grupos de los que es miembro el usuario para posteriormente insertar los nuevos" & ex.Message)
                        salida = salida & vbCrLf & "Error por falta de permisos: al eliminar los grupos de los que es miembro el equipo para posteriormente insertar los nuevos" & ex.Message

                    Catch ex As Exception When DN = ""
                        MsgBox("Error el Grupo al que se quiere añadir el usuario no pude ser vacío " & ex.Message)
                        salida = salida & vbCrLf & "Error: el Grupo al que se quiere añadir el usuario no pude ser vacío " & ex.Message

                    Catch ex As FieldAccessException
                        MsgBox("No tiene permisos para añadir el usuario a uno de los grupos " & ex.Message)
                        salida = salida & vbCrLf & "Error: el Grupo al que se quiere añadir el usuario no pude ser vacío " & ex.Message

                    Catch ex As Exception
                        MsgBox("Error al añadir el usuario al grupo " & logN)
                        salida = salida & vbCrLf & "Error: al añadir el usuario al grupo " & grupos(i) & ex.Message
                    End Try
                    'End If

                Next
            End If
        Catch ex As Exception
            MsgBox("Error al copiar los grupos de los que es miembro " & ex.Message)
            salida = salida & vbCrLf & "Error al copiar los grupos de los que es miembro " & ex.Message
        End Try
        '-----------------------------------Managed By---------------------------------------------------------------------
        '--------------------------------------------------------------------------------------------------------------------
        Try
            If Not groupClass.GetGroupProperty(Grupo, "managedBy") = "" Then
                groupClass.SetGroupProperty(Nnombre, "managedBy", groupClass.GetGroupProperty(Grupo, "managedBy"))
            End If
        Catch ex As Exception
            MsgBox("Error al establecer el administrador del grupo " & ex.Message)
            salida = salida & vbCrLf & "Error: al establecer el administrador del grupo " & ex.Message

        End Try

        '-----------------------------------Security---------------------------------------------------------------------
        '--------------------------------------------------------------------------------------------------------------------
        MailClass.CopyAces(Grupo, Nnombre)
        salida = salida & vbCrLf & "usuario copiado"
        GenericClass.Log(My.Application.Info.DirectoryPath & "\Log", salida, False)

            Else
                MsgBox("Copia de " & Grupo & "ya existe")
            End If
        End If
    End Sub

    Public Shared Function GetGroupList() As System.Collections.Generic.IList(Of String)
        '==========================================================================
        ' Name:     GetGroupList
        ' Purpose:  Devolver un array con todos los nombres de los grupos del directorio activo
        ' Input:    None
        ' Return:   1 array: Array de tipo string con los nombres de los grupos
        ' Calls:    None
        '==========================================================================

        'Buscar todos los objetos de tipo grupo del directorio activo 
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()
        Dim oresult As SearchResult
        Dim lNewList As New List(Of String)
        Dim user As New DirectoryEntry
        Dim logonN As String
        If Not results.Count = 0 Then
            For Each oresult In results
                Try
                    user = oresult.GetDirectoryEntry
                    'Para cada grupo añadir su nombre al array
                    logonN = user.Properties("sAMAccountName").Value.ToString
                    lNewList.Add(logonN)
                Catch e As FormatException
                    MsgBox("Error al acceder al nombre del grupo al crear una lista de todos los que hay en el directorio activo, Error de formato del dato al obtener la lista de los grupos del directorio activo  " & e.Message)
                Catch e As AccessViolationException
                    MsgBox("Error al acceder al nombre del grupo al crear una lista de todos los que hay en el directorio activo, No tienes permisos para acceder a este dato de " & e.Message)
                Catch e As ArgumentOutOfRangeException
                    MsgBox("Error al acceder al nombre del grupo al crear una lista de todos los que hay en el directorio activo, Se ha intentado obtener un dato con un índice fuera de rango " & e.Message)
                Catch e As OperationCanceledException
                    MsgBox("Error al acceder al nombre del grupo al crear una lista de todos los que hay en el directorio activo, Se está intentando obtener un dato involucrado en una operación cancelada " & e.Message)
                Catch e As OperationAbortedException
                    MsgBox("Error al acceder al nombre del grupo al crear una lista de todos los que hay en el directorio activo, Se está intentando obtener un dato involucrado en una operación abortada " & e.Message)
                Catch e As OutOfMemoryException
                    MsgBox("Error al acceder al nombre del grupo al crear una lista de todos los que hay en el directorio activo, La memoria está totalmente ocupada " & e.Message)
                Catch e As FieldAccessException
                    MsgBox("Error al acceder al nombre del grupo al crear una lista de todos los que hay en el directorio activo, Error al acceder a al ubicación física del dato " & e.Message)
                Catch e As InsufficientMemoryException
                    MsgBox("Error al acceder al nombre del grupo al crear una lista de todos los que hay en el directorio activo, La memoria está totalmente ocupada " & e.Message)
                Catch e As MissingFieldException
                    MsgBox("Error al acceder al nombre del grupo al crear una lista de todos los que hay en el directorio activo, Error al acceder a al ubicación física del dato, no existe " & e.Message)
                End Try
            Next
        End If
        Return lNewList
    End Function

    Public Shared Function GetGroupProperty(ByVal UserName As String, ByVal UserProperty As String) As String
        '==========================================================================
        ' Name:     GetGroupProperty
        ' Purpose:  Devolver el valor de un atributo del grupo
        ' Input:    2 string: El nombre del grupo 'UserName', El nombre de la propiedad a devolver 'UserProperty'
        ' Return:   1 string: el atributo a devolver que será de tipo string
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & UserName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()
        Dim oresult As SearchResult
        If results.Count = 0 Then
            Return ""
        Else
            oresult = results.Item(0)
            'Devolver el valor del atributo
            Try
                If Not oresult.GetDirectoryEntry.Properties(UserProperty).Value Is Nothing Then
                    Return oresult.GetDirectoryEntry.Properties(UserProperty).Value.ToString
                Else
                    Return ""
                End If
            Catch e As FormatException
                MsgBox("Error al acceder a un atributo del grupo, Error de formato del dato a obtener en  " & UserProperty & e.Message)
            Catch e As AccessViolationException
                MsgBox("Error al acceder a un atributo del grupo,No tienes permisos para acceder a este dato de " & UserProperty & e.Message)
            Catch e As ArgumentOutOfRangeException
                MsgBox("Error al acceder a un atributo del grupo,Se ha intentado obtener un dato con un índice fuera de rango " & UserProperty & e.Message)
            Catch e As OperationCanceledException
                MsgBox("Error al acceder a un atributo del grupo,Se está intentando obtener un dato involucrado en una operación cancelada " & UserProperty & e.Message)
            Catch e As OperationAbortedException
                MsgBox("Error al acceder a un atributo del grupo,Se está intentando obtener un dato involucrado en una operación abortada " & UserProperty & e.Message)
            Catch e As OutOfMemoryException
                MsgBox("Error al acceder a un atributo del grupo,La memoria está totalmente ocupada " & UserProperty & e.Message)
            Catch e As FieldAccessException
                MsgBox("Error al acceder a un atributo del grupo,Error al acceder a al ubicación física del dato " & UserProperty & e.Message)
            Catch e As InsufficientMemoryException
                MsgBox("Error al acceder a un atributo del grupo,La memoria está totalmente ocupada " & UserProperty & e.Message)
            Catch e As MissingFieldException
                MsgBox("Error al acceder a un atributo del grupo,Error al acceder a al ubicación física del dato, no existe " & UserProperty & e.Message)
            End Try
        End If
    End Function

    Public Shared Function GetGroupPropertyRefresh(ByVal UserName As String, ByVal UserProperty As String) As String
        '==========================================================================
        ' Name:     GetGroupProperty
        ' Purpose:  Devolver el valor de un atributo del grupo refrescando antes la memoria
        ' Input:    2 string: El nombre del grupo 'UserName', El nombre de la propiedad a devolver 'UserProperty'
        ' Return:   1 string: el atributo a devolver que será de tipo string
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & UserName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()

        If results.Count = 0 Then
            Return ""
        Else
            'Refrescar la memoria y devolver el valor del atributo
            Dim res As New DirectoryEntry(results(0).Path)
            Try
                res.RefreshCache(New String() {UserProperty})

                If Not res.Properties(UserProperty).Value Is Nothing Then
                    Return res.Properties(UserProperty).Value.ToString
                Else
                    Return ""
                End If
            Catch e As FormatException
                MsgBox("Error al acceder a un atributo del grupo,Error de formato del dato a obtener en  " & UserProperty & e.Message)
            Catch e As AccessViolationException
                MsgBox("Error al acceder a un atributo del grupo,No tienes permisos para acceder a este dato de " & UserProperty & e.Message)
            Catch e As ArgumentOutOfRangeException
                MsgBox("Error al acceder a un atributo del grupo,Se ha intentado obtener un dato con un índice fuera de rango " & UserProperty & e.Message)
            Catch e As OperationCanceledException
                MsgBox("Error al acceder a un atributo del grupo,Se está intentando obtener un dato involucrado en una operación cancelada " & UserProperty & e.Message)
            Catch e As OperationAbortedException
                MsgBox("Error al acceder a un atributo del grupo,Se está intentando obtener un dato involucrado en una operación abortada " & UserProperty & e.Message)
            Catch e As OutOfMemoryException
                MsgBox("Error al acceder a un atributo del grupo,La memoria está totalmente ocupada " & UserProperty & e.Message)
            Catch e As FieldAccessException
                MsgBox("Error al acceder a un atributo del grupo,Error al acceder a al ubicación física del dato " & UserProperty & e.Message)
            Catch e As InsufficientMemoryException
                MsgBox("Error al acceder a un atributo del grupo,La memoria está totalmente ocupada " & UserProperty & e.Message)
            Catch e As MissingFieldException
                MsgBox("Error al acceder a un atributo del grupo,Error al acceder a al ubicación física del dato, no existe " & UserProperty & e.Message)
            End Try
        End If
    End Function

    Public Shared Function GetGroupPropertyObject(ByVal Grupo As String, ByVal Propiedad As String) As Object
        '==========================================================================
        ' Name:     GetGroupProperty
        ' Purpose:  Devolver el valor de un atributo del grupo de tipo objeto
        ' Input:    2 string: El nombre del grupo 'Grupo', El nombre de la propiedad a devolver 'Propiedad'
        ' Return:   1 oject: el atributo a devolver que será de tipo object
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & Grupo & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim lNewList As New List(Of String)
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()
        Dim oresult As SearchResult
        If results.Count = 0 Then
            Return 0
        Else
            oresult = results.Item(0)
            'Devolver el valor del atributo
            Try
                Return oresult.GetDirectoryEntry.Properties(Propiedad).Value
            Catch e As FormatException
                MsgBox("Error al acceder a un atributo del grupo,Error de formato del dato a obtener en  " & Propiedad & e.Message)
            Catch e As AccessViolationException
                MsgBox("Error al acceder a un atributo del grupo,No tienes permisos para acceder a este dato de " & Propiedad & e.Message)
            Catch e As ArgumentOutOfRangeException
                MsgBox("Error al acceder a un atributo del grupo,Se ha intentado obtener un dato con un índice fuera de rango " & Propiedad & e.Message)
            Catch e As OperationCanceledException
                MsgBox("Error al acceder a un atributo del grupo,Se está intentando obtener un dato involucrado en una operación cancelada " & Propiedad & e.Message)
            Catch e As OperationAbortedException
                MsgBox("Error al acceder a un atributo del grupo,Se está intentando obtener un dato involucrado en una operación abortada " & Propiedad & e.Message)
            Catch e As OutOfMemoryException
                MsgBox("Error al acceder a un atributo del grupo,La memoria está totalmente ocupada " & Propiedad & e.Message)
            Catch e As FieldAccessException
                MsgBox("Error al acceder a un atributo del grupo,Error al acceder a al ubicación física del dato " & Propiedad & e.Message)
            Catch e As InsufficientMemoryException
                MsgBox("Error al acceder a un atributo del grupo,La memoria está totalmente ocupada " & Propiedad & e.Message)
            Catch e As MissingFieldException
                MsgBox("Error al acceder a un atributo del grupo,Error al acceder a al ubicación física del dato, no existe " & Propiedad & e.Message)
            End Try
        End If

    End Function

    Public Shared Function GetGrouplogonN(ByVal UserDN As String) As String
        '==========================================================================
        ' Name:     GetGrouplogonN
        ' Purpose:  Devolver el valor del atributo sAMAccountName del grupo 
        ' Input:    1 string: El valor del atributo distinguishedName del grupo 'UserDN'
        ' Return:   1 string: el atributo sAMAccountname es de tipo string
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (distinguishedName=" & UserDN & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()
        Dim oresult As SearchResult
        If results.Count = 0 Then
            Return "No hay Datos"
        Else
            'Devolver el sAMAccountName del grupo
            Try
                oresult = results.Item(0)
                If Not oresult.GetDirectoryEntry.Properties("sAMAccountName").Value Is Nothing Then
                    Return oresult.GetDirectoryEntry.Properties("sAMAccountName").Value.ToString
                Else
                    Return "No hay Datos"
                End If
            Catch e As FormatException
                MsgBox("Error al acceder al nombre de login del grupo, Error de formato del nombre de login a obtener en  " & UserDN & e.Message)
            Catch e As AccessViolationException
                MsgBox("Error al acceder al nombre de login del grupo, No tienes permisos para acceder al nombre de login de " & UserDN & e.Message)
            Catch e As ArgumentOutOfRangeException
                MsgBox("Error al acceder al nombre de login del grupo, Se ha intentado obtener un dato con un índice fuera de rango " & UserDN & e.Message)
            Catch e As OperationCanceledException
                MsgBox("Error al acceder al nombre de login del grupo, Se está intentando obtener un dato involucrado en una operación cancelada " & UserDN & e.Message)
            Catch e As OperationAbortedException
                MsgBox("Error al acceder al nombre de login del grupo, Se está intentando obtener un dato involucrado en una operación abortada " & UserDN & e.Message)
            Catch e As OutOfMemoryException
                MsgBox("Error al acceder al nombre de login del grupo, La memoria está totalmente ocupada " & UserDN & e.Message)
            Catch e As FieldAccessException
                MsgBox("Error al acceder al nombre de login del grupo, Error al acceder a al ubicación física del dato " & UserDN & e.Message)
            Catch e As InsufficientMemoryException
                MsgBox("Error al acceder al nombre de login del grupo, La memoria está totalmente ocupada " & UserDN & e.Message)
            Catch e As MissingFieldException
                MsgBox("Error al acceder al nombre de login del grupo, Error al acceder a al ubicación física del dato, no existe " & UserDN & e.Message)
            End Try
        End If
    End Function

    Public Shared Function GetGroupGroups(ByVal GName As String) As List(Of String)
        '==========================================================================
        ' Name:     GetGroupGroups
        ' Purpose:  Devolver un array con todos los nombres de los grupos de los que el grupo es miembro 
        ' Input:    1 string: El nombre del grupo 'GName'
        ' Return:   1 array: Array de strings con los nombres de los grupos de los que es miembro el grupo
        ' Calls:    None
        '==========================================================================

        'Obtener el valor del atributodistinguishedName del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & GName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim result As SearchResultCollection = deSearch.FindAll()
        Dim User As New DirectoryEntry(result(0).Path)
        Dim CNUSer As String = User.Properties("distinguishedName").Value.ToString

        'Buscar todos los grupos que tienen el distinguishedName del grupo como miembro
        Dim dire As DirectoryEntry = New DirectoryEntry()
        dire.Username = Settings.adminLogon
        dire.Password = Settings.adminPassword
        dire.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch2 As DirectorySearcher = New DirectorySearcher(dire)
        deSearch2.Filter = "(&(objectClass=group) (member=" & CNUSer & "))"
        deSearch2.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch2.FindAll()
        Dim resultado As SearchResult

        Dim grupos As New List(Of String)
        Dim i As Integer = 1
        For Each resultado In results
            'Para cada grupo encontrado se añade su nobre al array
            Try
                Dim Grupo As New DirectoryEntry(resultado.Path)
                grupos.Add(Grupo.Properties("sAMAccountName").Value.ToString)
            Catch e As FormatException
                MsgBox("Error al acceder a los grupos de los que es miembro el grupo, Error de formato del nombre de login a obtener en  " & GName & e.Message)
            Catch e As AccessViolationException
                MsgBox("Error al acceder a los grupos de los que es miembro el grupo, No tienes permisos para acceder al nombre de login de " & GName & e.Message)
            Catch e As ArgumentOutOfRangeException
                MsgBox("Error al acceder a los grupos de los que es miembro el grupo, Se ha intentado obtener un dato con un índice fuera de rango " & GName & e.Message)
            Catch e As OperationCanceledException
                MsgBox("Error al acceder a los grupos de los que es miembro el grupo, Se está intentando obtener un dato involucrado en una operación cancelada " & GName & e.Message)
            Catch e As OperationAbortedException
                MsgBox("Error al acceder a los grupos de los que es miembro el grupo, Se está intentando obtener un dato involucrado en una operación abortada " & GName & e.Message)
            Catch e As OutOfMemoryException
                MsgBox("Error al acceder a los grupos de los que es miembro el grupo, La memoria está totalmente ocupada " & GName & e.Message)
            Catch e As FieldAccessException
                MsgBox("Error al acceder a los grupos de los que es miembro el grupo, Error al acceder a al ubicación física del dato " & GName & e.Message)
            Catch e As InsufficientMemoryException
                MsgBox("Error al acceder a los grupos de los que es miembro el grupo, La memoria está totalmente ocupada " & GName & e.Message)
            Catch e As MissingFieldException
                MsgBox("Error al acceder a los grupos de los que es miembro el grupo, Error al acceder a al ubicación física del dato, no existe " & GName & e.Message)
            End Try
        Next

        Return grupos

    End Function

    Public Shared Function GetGroupMembers(ByVal GName As String) As List(Of String)
        '==========================================================================
        ' Name:     GetGroupGroups
        ' Purpose:  Devolver un array con todos los nombres de miembros del grupo
        ' Input:    1 string: El nombre del grupo 'GName'
        ' Return:   1 array: Array de strings con los nombres de los miembros del grupo
        ' Calls:    None
        '==========================================================================

        'Obtener el valor del atributo distinguishedName del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & GName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim result As SearchResultCollection = deSearch.FindAll()
        Dim grupos As New List(Of String)
        If result.Count > 0 Then
            Dim User As New DirectoryEntry(result(0).Path)
            Dim CNUSer As String = User.Properties("distinguishedName").Value.ToString


            'Buscar los objetos miembros del grupo
            Dim dire As DirectoryEntry = New DirectoryEntry()
            dire.Username = Settings.adminLogon
            dire.Password = Settings.adminPassword
            dire.AuthenticationType = AuthenticationTypes.Delegation
            Dim deSearch2 As DirectorySearcher = New DirectorySearcher(dire)
            deSearch2.Filter = "(&(memberOf=" & CNUSer & "))"
            deSearch2.SearchScope = SearchScope.Subtree
            Dim results As SearchResultCollection = deSearch2.FindAll()
            Dim resultado As SearchResult

            If results.Count > 0 Then
                Dim i As Integer = 1
                'Para cada objeto añadir su sAMAccountName al array
                For Each resultado In results
                    Try
                        Dim Grupo As New DirectoryEntry(resultado.Path)
                        Grupo.Username = Settings.adminLogon
                        Grupo.Password = Settings.adminPassword
                        Grupo.AuthenticationType = AuthenticationTypes.Delegation
                        grupos.Add(Grupo.Properties("sAMAccountName").Value.ToString)
                    Catch e As FormatException
                        MsgBox("Error al acceder a los miembros del grupo, Error de formato del nombre de login al obtener los miembros del grupo  " & GName & e.Message)
                    Catch e As AccessViolationException
                        MsgBox("Error al acceder a los miembros del grupo, No tienes permisos para acceder a los miembros de " & GName & e.Message)
                    Catch e As ArgumentOutOfRangeException
                        MsgBox("Error al acceder a los miembros del grupo, Se ha intentado obtener un dato con un índice fuera de rango " & GName & e.Message)
                    Catch e As OperationCanceledException
                        MsgBox("Error al acceder a los miembros del grupo, Se está intentando obtener un dato involucrado en una operación cancelada " & GName & e.Message)
                    Catch e As OperationAbortedException
                        MsgBox("Error al acceder a los miembros del grupo, Se está intentando obtener un dato involucrado en una operación abortada " & GName & e.Message)
                    Catch e As OutOfMemoryException
                        MsgBox("Error al acceder a los miembros del grupo, La memoria está totalmente ocupada " & GName & e.Message)
                    Catch e As FieldAccessException
                        MsgBox("Error al acceder a los miembros del grupo, Error al acceder a al ubicación física del dato " & GName & e.Message)
                    Catch e As InsufficientMemoryException
                        MsgBox("Error al acceder a los miembros del grupo, La memoria está totalmente ocupada " & GName & e.Message)
                    Catch e As MissingFieldException
                        MsgBox("Error al acceder a los miembros del grupo, Error al acceder a al ubicación física del dato, no existe " & GName & e.Message)
                    End Try
                Next
            End If
            
        Else
            MsgBox("no encuentro grupo")
        End If
        Return grupos

    End Function

    Public Shared Function GetGroupPropertyBool(ByVal GName As String, ByVal GProperty As String) As Boolean
        '==========================================================================
        ' Name:     GetGroupPropertyBool
        ' Purpose:  Devolver el valor de un atributo de tipo booleano de un grupo
        ' Input:    2 string: El nombre del grupo 'GName', El nombre del atributo 'GProperty'
        ' Return:   1 boolan: El valor del atributo
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & GName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()
        Dim oresult As SearchResult
        If results.Count = 0 Then
            MsgBox("Vacio")
        Else
            oresult = results.Item(0)
            'Si existe el grupo devolver el valor del atributo
            Try
                Return oresult.GetDirectoryEntry.Properties(GProperty).Value
            Catch e As FormatException
                MsgBox("Error al acceder a un atributo del grupo, Error de formato del dato a obtener en  " & GProperty & e.Message)
            Catch e As AccessViolationException
                MsgBox("Error al acceder a un atributo del grupo, No tienes permisos para acceder a este dato de " & GProperty & e.Message)
            Catch e As ArgumentOutOfRangeException
                MsgBox("Error al acceder a un atributo del grupo, Se ha intentado obtener un dato con un índice fuera de rango " & GProperty & e.Message)
            Catch e As OperationCanceledException
                MsgBox("Error al acceder a un atributo del grupo, Se está intentando obtener un dato involucrado en una operación cancelada " & GProperty & e.Message)
            Catch e As OperationAbortedException
                MsgBox("Error al acceder a un atributo del grupo, Se está intentando obtener un dato involucrado en una operación abortada " & GProperty & e.Message)
            Catch e As OutOfMemoryException
                MsgBox("Error al acceder a un atributo del grupo, La memoria está totalmente ocupada " & GProperty & e.Message)
            Catch e As FieldAccessException
                MsgBox("Error al acceder a un atributo del grupo, Error al acceder a un atributo del grupo, Error al acceder a al ubicación física del dato " & GProperty & e.Message)
            Catch e As InsufficientMemoryException
                MsgBox("Error al acceder a un atributo del grupo, La memoria está totalmente ocupada " & GProperty & e.Message)
            Catch e As MissingFieldException
                MsgBox("Error al acceder a un atributo del grupo, Error al acceder a al ubicación física del dato, no existe " & GProperty & e.Message)
            End Try
        End If
    End Function

    Public Shared Function GetGroupPropertyInteger(ByVal GName As String, ByVal GProperty As String) As Integer
        '==========================================================================
        ' Name:     GetGroupPropertyInteger
        ' Purpose:  Devolver el valor de un atributo de tipo integer de un grupo
        ' Input:    2 string: El nombre del grupo 'GName', El nombre del atributo 'GProperty'
        ' Return:   1 integer: El valor del atributo
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & GName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()
        'Dim oresult As SearchResult
        If results.Count = 0 Then
            MsgBox("Error obteniendo dato")
            Return -1
        Else
            'Si existe el grupo devolver el valor del atributo
            Dim res As New DirectoryEntry(results(0).Path)
            Try
                Return res.Properties(GProperty).Value
                res.CommitChanges()
            Catch e As FormatException
                MsgBox("Error al acceder a un atributo del grupo, Error de formato del dato a obtener en  " & GProperty & e.Message)
            Catch e As AccessViolationException
                MsgBox("Error al acceder a un atributo del grupo, No tienes permisos para acceder a este dato de " & GProperty & e.Message)
            Catch e As ArgumentOutOfRangeException
                MsgBox("Error al acceder a un atributo del grupo, Se ha intentado obtener un dato con un índice fuera de rango " & GProperty & e.Message)
            Catch e As OperationCanceledException
                MsgBox("Error al acceder a un atributo del grupo, Error al acceder a un atributo del grupo, Se está intentando obtener un dato involucrado en una operación cancelada " & GProperty & e.Message)
            Catch e As OperationAbortedException
                MsgBox("Error al acceder a un atributo del grupo, Se está intentando obtener un dato involucrado en una operación abortada " & GProperty & e.Message)
            Catch e As OutOfMemoryException
                MsgBox("Error al acceder a un atributo del grupo, La memoria está totalmente ocupada " & GProperty & e.Message)
            Catch e As FieldAccessException
                MsgBox("Error al acceder a un atributo del grupo, Error al acceder a al ubicación física del dato " & GProperty & e.Message)
            Catch e As InsufficientMemoryException
                MsgBox("Error al acceder a un atributo del grupo, La memoria está totalmente ocupada " & GProperty & e.Message)
            Catch e As MissingFieldException
                MsgBox("Error al acceder a un atributo del grupo, Error al acceder a al ubicación física del dato, no existe " & GProperty & e.Message)
            End Try
        End If
    End Function

    Public Shared Function GetGroupPropertyOIdentifier(ByVal GName As String, ByVal GProperty As String) As String
        '==========================================================================
        ' Name:     GetGroupPropertyInteger
        ' Purpose:  Devolver el valor de un atributo de tipo OIdentifier de un grupo
        ' Input:    2 string: El nombre del grupo 'GName', El nombre del atributo 'GProperty'
        ' Return:   1 string: El valor del atributo
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & GName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()

        If results.Count = 0 Then
            Return ""
        Else

            Dim res As New DirectoryEntry(results(0).Path)
            'Si existe el grupo devolver el valor del atributo
            Try
                If Not res.Properties(GProperty).Value Is Nothing Then
                    Dim sal() As Object = res.Properties(GProperty).Value
                    Return sal(sal.Length - 1).ToString
                Else
                    Return ""
                End If
            Catch e As FormatException
                MsgBox("Error al acceder a un atributo del grupo, Error de formato del dato a obtener en  " & GProperty & e.Message)
            Catch e As AccessViolationException
                MsgBox("Error al acceder a un atributo del grupo, No tienes permisos para acceder a este dato de " & GProperty & e.Message)
            Catch e As ArgumentOutOfRangeException
                MsgBox("Error al acceder a un atributo del grupo, Se ha intentado obtener un dato con un índice fuera de rango " & GProperty & e.Message)
            Catch e As OperationCanceledException
                MsgBox("Error al acceder a un atributo del grupo, Se está intentando obtener un dato involucrado en una operación cancelada " & GProperty & e.Message)
            Catch e As OperationAbortedException
                MsgBox("Error al acceder a un atributo del grupo, Se está intentando obtener un dato involucrado en una operación abortada " & GProperty & e.Message)
            Catch e As OutOfMemoryException
                MsgBox("Error al acceder a un atributo del grupo, La memoria está totalmente ocupada " & GProperty & e.Message)
            Catch e As FieldAccessException
                MsgBox("Error al acceder a un atributo del grupo, Error al acceder a al ubicación física del dato " & GProperty & e.Message)
            Catch e As InsufficientMemoryException
                MsgBox("Error al acceder a un atributo del grupo, La memoria está totalmente ocupada " & GProperty & e.Message)
            Catch e As MissingFieldException
                MsgBox("Error al acceder a un atributo del grupo, Error al acceder a al ubicación física del dato, no existe " & GProperty & e.Message)
            End Try
        End If
    End Function

    Public Shared Function GetGroupPropertyLargeInterval(ByVal GName As String, ByVal GProperty As String) As String
        '==========================================================================
        ' Name:     GetGroupPropertyLargeInterval
        ' Purpose:  Devolver el valor de un atributo de tipo LargeInterval de un grupo
        ' Input:    2 string: El nombre del grupo 'GName', El nombre del atributo 'GProperty'
        ' Return:   1 string: El valor del atributo
        ' Calls:    ToInt64 y GetBytes de la clase BitConverter
        '==========================================================================

        'Obtener el DirectoryEntry del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & GName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()

        If results.Count = 0 Then
            Return ""
        Else
            Dim res As New DirectoryEntry(results(0).Path)

            Dim low As Int32
            Dim high As Int32
            Dim valBytes(7) As Byte
            'Obtener el dato mediante la interfaz IADsLargeInteger
            Dim longInt As IADsLargeInteger = CType(res.Properties(GProperty).Value, IADsLargeInteger)
            low = longInt.LowPart
            high = longInt.HighPart

            'Pasar los int32 a bytes
            BitConverter.GetBytes(low).CopyTo(valBytes, 0)
            BitConverter.GetBytes(high).CopyTo(valBytes, 4)

            'Unirlos en un int64
            Dim salida As Int64 = BitConverter.ToInt64(valBytes, 0)
            Try
                If Not res.Properties(GProperty).Value Is Nothing Then
                    'Devolver el nuevo dato
                    Return salida
                Else
                    Return ""
                End If
            Catch e As FormatException
                MsgBox("Error al acceder a un atributo del grupo, Error de formato del dato a obtener en  " & GProperty & e.Message)
            Catch e As AccessViolationException
                MsgBox("Error al acceder a un atributo del grupo, No tienes permisos para acceder a este dato de " & GProperty & e.Message)
            Catch e As ArgumentOutOfRangeException
                MsgBox("Error al acceder a un atributo del grupo, Se ha intentado obtener un dato con un índice fuera de rango " & GProperty & e.Message)
            Catch e As OperationCanceledException
                MsgBox("Error al acceder a un atributo del grupo, Se está intentando obtener un dato involucrado en una operación cancelada " & GProperty & e.Message)
            Catch e As OperationAbortedException
                MsgBox("Error al acceder a un atributo del grupo, Se está intentando obtener un dato involucrado en una operación abortada " & GProperty & e.Message)
            Catch e As OutOfMemoryException
                MsgBox("Error al acceder a un atributo del grupo, La memoria está totalmente ocupada " & GProperty & e.Message)
            Catch e As FieldAccessException
                MsgBox("Error al acceder a un atributo del grupo, Error al acceder a al ubicación física del dato " & GProperty & e.Message)
            Catch e As InsufficientMemoryException
                MsgBox("Error al acceder a un atributo del grupo, La memoria está totalmente ocupada " & GProperty & e.Message)
            Catch e As MissingFieldException
                MsgBox("Error al acceder a un atributo del grupo, Error al acceder a al ubicación física del dato, no existe " & GProperty & e.Message)
            End Try
        End If
    End Function

    Public Shared Function GetGrouplogonNCN(ByVal UserDN As String) As String
        '==========================================================================
        ' Name:     GetGrouplogonNCN
        ' Purpose:  Devolver el valor del atributo sAMAccountName de un grupo
        ' Input:    1 string: El atributo cn del grupo 'UserDN'
        ' Return:   1 string: El valor del atributo sAMAccountName
        ' Calls:    None
        '==========================================================================

        'Obtener el directoyEntry del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (cn=" & UserDN & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()
        Dim oresult As SearchResult
        If results.Count = 0 Then
            Return "No hay Datos"
        Else
            oresult = results.Item(0)
            'Devolver su atributo sAMAccountName
            Try
                If Not oresult.GetDirectoryEntry.Properties("sAMAccountName").Value Is Nothing Then
                    Return oresult.GetDirectoryEntry.Properties("sAMAccountName").Value.ToString
                Else
                    Return "No hay Datos"
                End If
            Catch e As FormatException
                MsgBox("Error al acceder al nombre canónico del grupo, Error de formato del nombre de login a obtener  " & e.Message)
            Catch e As AccessViolationException
                MsgBox("Error al acceder al nombre canónico del grupo, No tienes permisos para acceder al nombre de login " & e.Message)
            Catch e As ArgumentOutOfRangeException
                MsgBox("Error al acceder al nombre canónico del grupo, Se ha intentado obtener un dato con un índice fuera de rango " & e.Message)
            Catch e As OperationCanceledException
                MsgBox("Error al acceder al nombre canónico del grupo, Se está intentando obtener un dato involucrado en una operación cancelada " & e.Message)
            Catch e As OperationAbortedException
                MsgBox("Error al acceder al nombre canónico del grupo, Se está intentando obtener un dato involucrado en una operación abortada " & e.Message)
            Catch e As OutOfMemoryException
                MsgBox("Error al acceder al nombre canónico del grupo, La memoria está totalmente ocupada " & e.Message)
            Catch e As FieldAccessException
                MsgBox("Error al acceder al nombre canónico del grupo, Error al acceder a al ubicación física del dato " & e.Message)
            Catch e As InsufficientMemoryException
                MsgBox("Error al acceder al nombre canónico del grupo, La memoria está totalmente ocupada " & e.Message)
            Catch e As MissingFieldException
                MsgBox("Error al acceder al nombre canónico del grupo, Error al acceder a al ubicación física del dato, no existe " & e.Message)
            End Try
        End If
    End Function

    Public Shared Function CargarAdminGroup() As List(Of String)
        '==========================================================================
        ' Name:     CargarAdminGroup
        ' Purpose:  Devolver un array con las direcciones de los grupos administrativos del directorio activo
        ' Input:    None
        ' Return:   1 array: El valor del atributo sAMAccountName
        ' Calls:    None
        '==========================================================================

        Dim Organizations As SearchResultCollection
        Dim Organization As SearchResult
        Dim AdminGroups As SearchResultCollection
        Dim AdminGroup As SearchResult
        Dim lista As New List(Of String)

        'Obtener DirectoryEntry de la raiz del directorio activo  y buscar sus Contenedores Exchange
        Dim RootDSE As New DirectoryEntry("LDAP://RootDSE")
        RootDSE.Username = Settings.adminLogon
        RootDSE.Password = Settings.adminPassword
        RootDSE.AuthenticationType = AuthenticationTypes.Delegation
        Dim RootPath As String = "LDAP://" + RootDSE.Properties("configurationNamingContext").Value
        Dim ConfigContainer As New DirectoryEntry(RootPath)
        ConfigContainer.Username = Settings.adminLogon
        ConfigContainer.Password = Settings.adminPassword
        ConfigContainer.AuthenticationType = AuthenticationTypes.Delegation
        Dim ConfigSearcher As New DirectorySearcher(ConfigContainer)
        ConfigSearcher.Filter = "(&(objectCategory=msExchOrganizationContainer))"
        ConfigSearcher.SearchScope = SearchScope.Subtree
        Organizations = ConfigSearcher.FindAll
        For Each Organization In Organizations
            'Buscar sus ExchangeAdminGroups y añadir al array su dirección
            Dim ConfigContainer2 As New DirectoryEntry(Organization.Path)
            ConfigContainer2.Username = Settings.adminLogon
            ConfigContainer2.Password = Settings.adminPassword
            Dim ConfigSearcher2 As New DirectorySearcher(ConfigContainer2)
            ConfigSearcher2.Filter = "(&(objectCategory=msExchAdminGroup))"
            ConfigSearcher2.SearchScope = SearchScope.Subtree
            AdminGroups = ConfigSearcher2.FindAll
            For Each AdminGroup In AdminGroups
                Try
                    lista.Add(Organization.GetDirectoryEntry.Properties("cn").Value.ToString & "/" & AdminGroup.GetDirectoryEntry.Properties("cn").Value.ToString)
                Catch e As FormatException
                    MsgBox("Error al acceder al grupo administrativo del correo del grupo, Error de formato al obtener los grupos administrativos del directorio activo  " & e.Message)
                Catch e As AccessViolationException
                    MsgBox("Error al acceder al grupo administrativo del correo del grupo, No tienes permisos para acceder a los grupos administrativos del directorio activo " & e.Message)
                Catch e As ArgumentOutOfRangeException
                    MsgBox("Error al acceder al grupo administrativo del correo del grupo, Se ha intentado obtener un dato con un índice fuera de rango " & e.Message)
                Catch e As OperationCanceledException
                    MsgBox("Error al acceder al grupo administrativo del correo del grupo, Se está intentando obtener un dato involucrado en una operación cancelada " & e.Message)
                Catch e As OperationAbortedException
                    MsgBox("Error al acceder al grupo administrativo del correo del grupo, Se está intentando obtener un dato involucrado en una operación abortada " & e.Message)
                Catch e As OutOfMemoryException
                    MsgBox("Error al acceder al grupo administrativo del correo del grupo, La memoria está totalmente ocupada " & e.Message)
                Catch e As FieldAccessException
                    MsgBox("Error al acceder al grupo administrativo del correo del grupo, Error al acceder a al ubicación física del dato " & e.Message)
                Catch e As InsufficientMemoryException
                    MsgBox("Error al acceder al grupo administrativo del correo del grupo, La memoria está totalmente ocupada " & e.Message)
                Catch e As MissingFieldException
                    MsgBox("Error al acceder al grupo administrativo del correo del grupo, Error al acceder a al ubicación física del dato, no existe " & e.Message)
                End Try
            Next
        Next
        Return lista
    End Function
    '----------------------------------------------------------Set---------------------------------------------------------------------------
    '----------------------------------------------------------------------------------------------------------------------------------------
    Public Shared Sub SetGroupProperty(ByVal GName As String, ByVal GProperty As String, ByVal Datos As String)
        '==========================================================================
        ' Name:     SetGroupProperty
        ' Purpose:  Establecer el valor de un atributo de tipo string de un grupo
        ' Input:    3 string: El nombre del grupo 'GName', El nombre del atributo 'GProperty'
        '           valor a dar al atributo'Datos'
        ' Return:   None
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & GName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()

        If results.Count = 0 Then
            MsgBox("Error cargando" & GProperty)
        Else
            'Darle al atributo el valor que tenemos en los parámetros
            Dim oresult As New DirectoryEntry(results(0).Path)
            oresult.Username = Settings.adminLogon
            oresult.Password = Settings.adminPassword
            oresult.AuthenticationType = AuthenticationTypes.Delegation
            oresult.Properties(GProperty).Value = Datos
            Try
                oresult.CommitChanges()
            Catch e As NullReferenceException
                MsgBox("Los datos han de tener algún valor para poder asignarlos a " & GProperty & e.Message)
            Catch e As FormatException
                MsgBox("Error de formato del dato a introducir en  " & GProperty & e.Message)
            Catch e As AccessViolationException
                MsgBox("No tienes permisos para asignar este dato a " & GProperty & e.Message)
            End Try
        End If

    End Sub

    Public Shared Sub SetGroupPropertyInsert(ByVal GName As String, ByVal GProperty As String, ByVal datos As String, ByVal indice As String)
        '==========================================================================
        ' Name:     SetGroupPropertyInsert
        ' Purpose:  Añadir un nuevo miembro a un atributo de tipo lista del grupo
        ' Input:    4 string: El nombre del grupo 'GName', El nombre del atributo 'GProperty'
        '           valor a dar al atributo'datos', El indice donde meter los datos 'indice'
        ' Return:   None
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del usuario
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & GName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()

        If results.Count = 0 Then
            MsgBox("Error obteniendo dato")
        Else
            'Añadir el nuevo miembro en la lista y actualizar el directoryEntry
            Dim res As New DirectoryEntry(results(0).Path)
            res.Username = Settings.adminLogon
            res.Password = Settings.adminPassword
            res.AuthenticationType = AuthenticationTypes.Delegation

            res.RefreshCache(New String() {GProperty})
            res.Properties(GProperty).Add(datos)

            Try
                res.CommitChanges()
            Catch e As AccessViolationException
                MsgBox("No tienes permisos para insertar objetos en " & GProperty & e.Message)
            Catch e As ArgumentException
                MsgBox("El argumento no tiene el formato adecuado para insertarlo en " & GProperty & e.Message)
            Catch e As ArgumentNullException
                MsgBox("No se puede insertar un objeto de tipo Null en " & GProperty & e.Message)
            Catch e As DuplicateNameException
                MsgBox("El objeto a insertar ya existe en " & GProperty & e.Message)
            Catch e As FormatException
                MsgBox("El objeto a insertar posee un formato no valido para " & GProperty & e.Message)
            Catch e As IndexOutOfRangeException
                MsgBox("El objeto a insertar no entra en el rango de " & GProperty & e.Message)
            End Try
            res.Close()
        End If
    End Sub

    Public Shared Sub SetGroupPropertyRemove(ByVal GName As String, ByVal GProperty As String)
        '==========================================================================
        ' Name:     SetGroupPropertyRemove
        ' Purpose:  Eliminar un atributo de tipo lista del grupo
        ' Input:    2 string: El nombre del grupo 'GName', El nombre del atributo 'GProperty'
        ' Return:   None
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & GName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()

        If results.Count = 0 Then
            MsgBox("Error obteniendo dato")
        Else
            'Eliminar el atributo y actualizar el directoryEntry del grupo
            Dim res As New DirectoryEntry(results(0).Path)
            res.Username = Settings.adminLogon
            res.Password = Settings.adminPassword
            res.AuthenticationType = AuthenticationTypes.Delegation

            res.Properties(GProperty).Clear()

            Try
                res.CommitChanges()
            Catch e As AccessViolationException
                MsgBox("No tiene permisos para eliminar los elementos de " & GProperty & e.Message)
            Catch e As ArgumentNullException
                MsgBox("No tiene permisos para eliminar los elementos de una propiedad que es nula " & e.Message)
            End Try
            res.Close()

        End If
    End Sub

    Public Shared Sub ClearGroupProperty(ByVal GName As String, ByVal GProperty As String)
        '==========================================================================
        ' Name:     ClearGroupProperty
        ' Purpose:  Eliminar un atributo de tipo lista del grupo
        ' Input:    2 string: El nombre del grupo 'GName', El nombre del atributo 'GProperty'
        ' Return:   None
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry del grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & GName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()
        If results.Count = 0 Then
            MsgBox("Error obteniendo datos")
        Else
            'Eliminar el atributo y actualizar el directoryEntry del grupo
            Dim res As New DirectoryEntry(results(0).Path)
            res.Properties(GProperty).Clear()
            res.Username = Settings.adminLogon
            res.Password = Settings.adminPassword
            res.AuthenticationType = AuthenticationTypes.Delegation

            Try
                res.CommitChanges()
            Catch e As AccessViolationException
                MsgBox("No tiene permisos para eliminar los elementos de " & GProperty & e.Message)
            Catch e As ArgumentNullException
                MsgBox("No tiene permisos para eliminar los elementos de una propiedad que es nula " & e.Message)
            End Try
            res.Close()
        End If
    End Sub

    Public Shared Sub SetGroupPropertyBool(ByVal GName As String, ByVal GProperty As String, ByVal Datos As Boolean)
        '==========================================================================
        ' Name:     SetGroupPropertyBool
        ' Purpose:  Establecer el valor de un atributo de tipo boolean del grupo 
        ' Input:    2 string: El nombre del grupo 'GName', El nombre del atributo 'GProperty'
        '           1 boolean: Los datos a asignar al atributo 'Datos'
        ' Return:   None
        ' Calls:    None
        '==========================================================================

        'Obtener el DirectoryEntry
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)
        deSearch.Filter = "(&(objectClass=group) (sAMAccountName=" & GName & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim results As SearchResultCollection = deSearch.FindAll()

        If results.Count = 0 Then
            MsgBox("Error cargando" & GProperty)
        Else
            'Asignar los datos al atributo
            Dim oresult As New DirectoryEntry(results(0).Path)
            oresult.Username = Settings.adminLogon
            oresult.Password = Settings.adminPassword
            oresult.AuthenticationType = AuthenticationTypes.Delegation

            oresult.Properties(GProperty).Value = Datos
            Try
                oresult.CommitChanges()
            Catch e As AccessViolationException
                MsgBox("Error, No tienes permisos para asignar un booleano a " & GProperty & e.Message)
            Catch e As ArgumentException
                MsgBox("Error, El argumento no es un booleano no se puede asignar a " & GProperty & e.Message)
            Catch e As ArgumentNullException
                MsgBox("Error, El argumento no es un booleano, es nulo , no se puede asignar a " & GProperty & e.Message)
            Catch e As FormatException
                MsgBox("Error, El argumento no es un booleano no se puede asignar a " & GProperty & e.Message)
            End Try
        End If
    End Sub

    '----------------------------------------------------------------------------------------------------------------------------------------
    '---------------------------------------------------------Mail--------------------------------------------------------------------------


    '----------------------------------------------------------------------------------------------------------------------------------------
    '---------------------------------------------------------Group/User--------------------------------------------------------------------------

    Public Shared Sub AddUserToGroup2(ByVal UserName As String, ByVal GroupName As String)
        '==========================================================================
        ' Name:     AddUserToGroup2
        ' Purpose:  Añade un usuario a la lista de miembros de un grupo 
        ' Input:    2 string: El nombre del grupo 'GroupName', El nombre de login del usuario 'UserName
        ' Return:   None
        ' Calls:    None
        '==========================================================================

        'Obtener DirectoryEntrys del usuario y el grupo
        Dim de As New DirectoryEntry("LDAP://" & Settings.ldapServerName & "/" & Settings.ADRoot)
        Dim deSearch As DirectorySearcher = New DirectorySearcher()
        deSearch.SearchRoot = de

        deSearch.Filter = "(&(objectClass=user) (sAMAccountName=" & UserName & "))"
        Dim resultsU As SearchResultCollection = deSearch.FindAll()
        Dim child As New DirectoryEntry(resultsU(0).Path)

        deSearch.Filter = "(&(objectClass=group) (cn=" & GroupName & "))"
        Dim results As SearchResultCollection = deSearch.FindAll()
        Dim isGroupMember As Boolean = False
        If results.Count > 0 Then

            Dim group As New DirectoryEntry(results(0).Path)
            group.Username = Settings.adminLogon
            group.Password = Settings.adminPassword
            group.AuthenticationType = AuthenticationTypes.Delegation
            'Buscar el usuario entre los miembros del grupo
            Dim members As Object = group.Invoke("Members", Nothing)
            For Each member As Object In CType(members, IEnumerable)
                Dim x As DirectoryEntry = New DirectoryEntry(member)
                x.Username = Settings.adminLogon
                x.Password = Settings.adminPassword
                x.AuthenticationType = AuthenticationTypes.Delegation
                Dim name As String = x.Name
                If name <> child.Name Then
                    isGroupMember = False
                Else
                    isGroupMember = True
                    Exit For
                End If
            Next member
            'Añadir el usuario si no es miembro todavía
            If (Not isGroupMember) Then
                group.Invoke("Add", New Object() {child.Path.ToString()})
            End If
            Try
                group.CommitChanges()
            Catch e As AccessViolationException
                MsgBox("No se tienen permisos para añadir al usuario al grupo " & GroupName & e.Message)
            Catch e As ArgumentNullException
                MsgBox("Ni grupo ni usuario puede ser nulo al añadir al usuario al grupo " & GroupName & e.Message)
            End Try

            group.Close()
        Else
            Console.WriteLine("No existe ese grupo")
        End If
        Return
    End Sub

    Public Shared Sub AddToGroup(ByVal Name As String, ByVal GroupName As String)
        '==========================================================================
        ' Name:     AddToGroup
        ' Purpose:  Añade un objeto a la lista de miembros de un grupo 
        ' Input:    2 string: El nombre del grupo 'GroupName', El nombre de login del objeto 'UserName
        ' Return:   None
        ' Calls:    GroupPropertyInsert de la clase GroupClass
        '==========================================================================

        'Mientras el grupo no sea DomainUsers, obtener DirectoryEntrys del usuario y el grupo
        Dim de As DirectoryEntry = New DirectoryEntry()
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher()
        deSearch.SearchRoot = de

        If Not GroupName = "Domain Users" Then
            deSearch.Filter = "((sAMAccountName=" & Name & "))"
            Dim resultsU As SearchResultCollection = deSearch.FindAll()
            Dim child As New DirectoryEntry(resultsU(0).Path)

            deSearch.Filter = "(&(objectClass=group) (cn=" & GroupName & "))"
            Dim results As SearchResultCollection = deSearch.FindAll()
            Dim isGroupMember As Boolean = False
            If results.Count > 0 Then
                'Si el usuario no es miembro del grupo se añade
                Dim group As New DirectoryEntry(results(0).Path)
                If Not group.Properties("member").Value.GetType = GetType(String) Then
                    If Not group.Properties("member").Contains(child.Properties("distinguishedName").Value.ToString) Then
                        group.RefreshCache(New String() {"member"})
                        group.Properties("member").Insert(group.Properties("member").Count, child.Properties("distinguishedName").Value.ToString)
                        group.CommitChanges()
                    End If
                Else
                    If Not group.Properties("member").Value.Contains(child.Properties("distinguishedName").Value.ToString) Then
                        group.RefreshCache(New String() {"member"})
                        group.Properties("member").Insert(1, child.Properties("distinguishedName").Value.ToString)
                        Try
                            group.CommitChanges()
                        Catch e As AccessViolationException
                            MsgBox("No se tienen permisos para añadir un miembro al grupo " & GroupName & e.Message)
                        Catch e As ArgumentNullException
                            MsgBox("Ni grupo ni el nuevo miembro puede ser nulo al añadir al usuario al grupo " & GroupName & e.Message)
                        End Try
                    End If
                End If
            Else
                Console.WriteLine("No existe ese grupo")
            End If
        End If
    End Sub

    Public Shared Sub RemoveUserMemberOfGroup(ByVal User As String, ByVal Group As String)
        '==========================================================================
        ' Name:     RemoveUserMemberOfGroup
        ' Purpose:  Elimina un usuario de la lista de miembros de un grupo 
        ' Input:    2 string: El nombre del grupo 'GroupName', El nombre de login del objeto 'UserName
        ' Return:   None
        ' Calls:    GroupPropertyInsert de la clase GroupClass
        '==========================================================================

        'Obtiene el DirectoryEntry del usuario y el grupo
        Dim de As New DirectoryEntry("LDAP://" & Settings.ldapServerName & "/" & Settings.ADRoot) '192.168.1.1/CN=Users;DC=xabi;DC=local")
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)

        Dim deSearchU As DirectorySearcher = New DirectorySearcher(de)
        deSearchU.Filter = "(&(objectClass=user) (sAMAccountName=" & User & "))"
        deSearchU.SearchScope = SearchScope.Subtree
        Dim resUC As SearchResultCollection = deSearchU.FindAll()
        Dim PUser As New DirectoryEntry(resUC(0).Path)
        Dim UserComplete As String = PUser.Properties("distinguishedName").Value.ToString

        deSearch.Filter = "(&(objectClass=group) (cn=" & Group & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim resC As SearchResultCollection = deSearch.FindAll()
        Dim res As SearchResult

        'Borra los miembros del atributo (previo guardado de los mismos en un array auxiliar) y añade todos los borrados excepto el usuario
        If resC.Count > 0 Then
            For Each res In resC
                Dim PGrupo As New DirectoryEntry(res.Path)
                PGrupo.Username = Settings.adminLogon
                PGrupo.Password = Settings.adminPassword
                PGrupo.AuthenticationType = AuthenticationTypes.Delegation

                PGrupo.RefreshCache(New String() {"member"})
                If Not PGrupo.Properties("member").Value.GetType = GetType(String) Then
                    Dim member() As Object = PGrupo.Properties("member").Value
                    Dim sal(member.Length - 1) As String
                    If Not member Is Nothing Then
                        PGrupo.Properties("member").Clear()
                        Dim j As Integer = 0
                        For i = 0 To member.Length - 1
                            If Not member(i).ToString = UserComplete Then
                                'PGrupo.Properties("member").Insert(i, member(i))
                                PGrupo.Properties("member").Add(member(i))
                                'j = j + 1
                            End If
                        Next
                    End If
                Else

                    PGrupo.Properties("member").Clear()

                End If
                    Try
                        PGrupo.CommitChanges()
                    Catch e As AccessViolationException
                        MsgBox("No se tienen permisos para eliminar el usuario como miembro del grupo " & Group & e.Message)
                    Catch e As ArgumentException
                        MsgBox("EL grupo o el usuario no tienen el formato adecuado para eliminar el usuario como miembro del grupo " & Group & e.Message)
                    Catch e As ArgumentNullException
                        MsgBox("No puede haber argumentos nulos para eliminar el usuario como miembro del grupo " & Group & e.Message)
                    Catch e As FormatException
                        MsgBox("EL grupo o el usuario no tienen el formato adecuado para eliminar el usuario como miembro del grupo " & Group & e.Message)
                    End Try
            Next
        Else
            MsgBox("No encontrado grupo")
        End If

    End Sub

    Public Shared Sub RemoveMemberOfGroup(ByVal User As String, ByVal Group As String)
        '==========================================================================
        ' Name:     RemoveMemberOfGroup
        ' Purpose:  Elimina un objeto de la lista de miembros de un grupo 
        ' Input:    2 string: El nombre del grupo 'GroupName', El nombre de login del objeto 'UserName
        ' Return:   None
        ' Calls:    GroupPropertyInsert de la clase GroupClass
        '==========================================================================

        'Obtiene el DirectoryEntry del usuario y el grupo
        Dim de As New DirectoryEntry("LDAP://" & Settings.ldapServerName & "/" & Settings.ADRoot) '192.168.1.1/CN=Users;DC=xabi;DC=local")
        de.Username = Settings.adminLogon
        de.Password = Settings.adminPassword
        de.AuthenticationType = AuthenticationTypes.Delegation
        Dim deSearch As DirectorySearcher = New DirectorySearcher(de)

        Dim deSearchU As DirectorySearcher = New DirectorySearcher(de)
        deSearchU.Filter = "(&(sAMAccountName=" & User & "))"
        deSearchU.SearchScope = SearchScope.Subtree
        Dim resUC As SearchResultCollection = deSearchU.FindAll()
        Dim PUser As New DirectoryEntry(resUC(0).Path)
        Dim UserComplete As String = PUser.Properties("distinguishedName").Value.ToString

        deSearch.Filter = "(&(objectClass=group) (cn=" & Group & "))"
        deSearch.SearchScope = SearchScope.Subtree
        Dim resC As SearchResultCollection = deSearch.FindAll()
        Dim res As SearchResult

        'Borra los miembros del atributo (previo guardado de los mismos en un array auxiliar) y añade todos los borrados excepto el usuario
        If resC.Count > 0 Then
            For Each res In resC
                Dim PGrupo As New DirectoryEntry(res.Path)
                PGrupo.Username = Settings.adminLogon
                PGrupo.Password = Settings.adminPassword
                PGrupo.AuthenticationType = AuthenticationTypes.Delegation

                PGrupo.RefreshCache(New String() {"member"})
                If Not PGrupo.Properties("member").Value.GetType = GetType(String) Then
                    Dim member() As Object = PGrupo.Properties("member").Value
                    Dim sal(member.Length - 1) As String
                    If Not member Is Nothing Then
                        PGrupo.Properties("member").Clear()
                        Dim j As Integer = 0
                        For i = 0 To member.Length - 1
                            If Not member(i).ToString = UserComplete Then
                                'PGrupo.Properties("member").Insert(i, member(i))
                                PGrupo.Properties("member").Add(member(i))
                                'j = j + 1
                            End If
                        Next
                    End If
                Else

                    PGrupo.Properties("member").Clear()

                End If
                Try
                    PGrupo.CommitChanges()
                Catch e As AccessViolationException
                    MsgBox("No se tienen permisos para eliminar el usuario como miembro del grupo " & Group & e.Message)
                Catch e As ArgumentException
                    MsgBox("EL grupo o el usuario no tienen el formato adecuado para eliminar el usuario como miembro del grupo " & Group & e.Message)
                Catch e As ArgumentNullException
                    MsgBox("No puede haber argumentos nulos para eliminar el usuario como miembro del grupo " & Group & e.Message)
                Catch e As FormatException
                    MsgBox("EL grupo o el usuario no tienen el formato adecuado para eliminar el usuario como miembro del grupo " & Group & e.Message)
                End Try
            Next
        Else
            MsgBox("No encontrado grupo")
        End If

    End Sub

    '--------------------------------------------------------------------------------------------------------------------------------------------
    '--------------------------------------------------------Interfaz----------------------------------------------------------------------------
    <ComImport(), Guid("9068270b-0939-11D1-8be1-00c04fd8d503")> _
    Public Interface IADsLargeInteger
        Property HighPart As Int32
        Property LowPart As Int32
    End Interface

End Class
