﻿Imports System
Imports System.Management
Imports System.IO
Imports System.Security.Principal
Imports impersonateclass
Imports System.Security.AccessControl



Public Class WMIClass


    Public Shared Function getSharedElements(ByVal server As String) As List(Of String)
        '==========================================================================
        ' Name:     getSharedElements
        ' Purpose:  Obtener una lista con los nombres de los recursos compartidos por un servidor
        ' Input:    1 String: El nombre del servidor 'server'
        ' Return:   1 array: array de strings en los que cada elemento es el nombre de un recurso compartido
        ' Calls:    None
        '==========================================================================

        Try
            Dim sharedE As New List(Of String)
            'crear la conexión impersonada con permisos de controlador de dominio
            Dim connection As New ConnectionOptions()
            connection.Username = Settings.adminLogon
            connection.Password = Settings.adminPassword
            connection.Impersonation = ImpersonationLevel.Impersonate

            'Establecer la carpeta desde donde buscar los recursos compartidos
            Dim scope As New ManagementScope( _
            "\\" & server & "\root\cimv2", connection)
            scope.Connect()

            'La sentencia SQL para buscarlos
            Dim query As New ObjectQuery( _
                "SELECT * FROM Win32_Share")

            'Realizar la llamada
            Dim searcher As New ManagementObjectSearcher(scope, query)

            'Para cada recurso compartido obtendio se añade su nombre a la lista
            For Each queryObj As ManagementObject In searcher.Get()
                Dim carpeta As String = queryObj.GetPropertyValue("Name").ToString
                sharedE.Add(carpeta)
            Next

            Return sharedE
        Catch err As ManagementException
            MessageBox.Show("An error occured while querying for WMI data: " _
                & err.Message)
        Catch unauthorizedErr As System.UnauthorizedAccessException

            MessageBox.Show("Connection error " & _
                "(user name or password might be incorrect): " & _
                unauthorizedErr.Message)
        End Try
    End Function

    Public Shared Function getWinUser(ByVal computer As String) As String
        '==========================================================================
        ' Name:     getSharedElements
        ' Purpose:  Obtener una lista con los nombres de los recursos compartidos por un servidor
        ' Input:    1 String: El nombre del servidor 'server'
        ' Return:   1 array: array de strings en los que cada elemento es el nombre de un recurso compartido
        ' Calls:    None
        '==========================================================================

        Dim resultados As New List(Of String)
        Dim nombre As String = ""
        Const HKEY_CURRENT_USER = &H80000001
        Try

            'Crear el objeto del que obtener los datos, obteniendo permisos de controlador de dominio
            'Dim oLocator As Object = CreateObject("WbemScripting.SWbemLocator")
            Dim strComputer As String = computer

            'Dim oService As Object = oLocator.ConnectServer(strComputer, "Root\DEFAULT", Settings.adminLogon, Settings.adminPassword)

            Dim oService As Object = GetObject("winmgmts:" _
              & "{impersonationLevel=impersonate}!\\" _
              & strComputer & "\root\cimv2")

            Dim colSessions
            colSessions = oService.ExecQuery _
    ("Select * from Win32_LogonSession Where LogonType = 0")

            For Each objSession In colSessions
                Dim colList
                colList = oService.ExecQuery("Associators of " _
         & "{Win32_LogonSession.LogonId=" & objSession.LogonId & "} " _
         & "Where AssocClass=Win32_LoggedOnUser Role=Dependent")

                For Each objItem In colList
                    'Dim int As Integer = objItem.LogonType
                    nombre = nombre & objItem.Domain & "\" & objItem.Name
                Next
            Next
            Return nombre

        Catch err As Exception
            MessageBox.Show("Error al intentar obtener el usuario actual de windows para el log: " _
                & err.Message)
        End Try
    End Function

    Public Shared Sub createFolder(ByVal Father As String, ByVal server As String, ByVal nombre As String)
        '==========================================================================
        ' Name:     createFolder
        ' Purpose:  Crear una nueva carpeta en un recurso compartido de un equipo remmoto
        ' Input:    3 String: El nombre del directorio en el que crear la nueva carpeta 'Father',
        '           El nombre del servidor 'server', El nombre de la nueva carpeta'nombre'
        ' Return:   None
        ' Calls:    None
        '==========================================================================

        Try
            'Se forma la dirección en la que crear la carpeta
            Dim path As String
            path = "\\" & server & "\" & Father

            If Directory.Exists(path) Then
                'Si en la dirección se comprueba que hay un directorio, se crea la nueva carpeta
                Directory.CreateDirectory(path & "\" & nombre)
            End If

        Catch err As ManagementException
            MessageBox.Show("An error occured while querying for WMI data: " _
                & err.Message)

        End Try
    End Sub

    Public Shared Sub createFolderArbol(ByVal dir As String, ByVal nombre As String)
        '==========================================================================
        ' Name:     createFolder
        ' Purpose:  Crear una nueva carpeta en un recurso compartido de un equipo remmoto
        ' Input:    2 String: La dirección en la que se creará el directorio 'dir',
        '           El nombre de la nueva carpeta'nombre'
        ' Return:   None
        ' Calls:    None
        '==========================================================================
        Try
            'Impersonar para obtener permisos de controlador de dominio
            Dim oImp1 As Impersonation
            oImp1 = New Impersonation()
            Dim partes() As String = Settings.adminLogon.Split("\")
            oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)

            Dim path As String

            path = dir
            'Si en la dirección hay un directorio
            If Directory.Exists(path) Then

                ' Evitar carpetas de sistema
                Dim objFSO, objFolder, colSubfolders, obj
                objFSO = CreateObject("Scripting.FileSystemObject")
                objFolder = objFSO.GetFolder(path)
                If (objFolder.Attributes > 21) Then
                    MsgBox("No se pueden modificar las carpertas de sistema")
                Else
                    'Crear la carpeta
                    Try

                        Directory.CreateDirectory(path & "\" & nombre)
                    Catch e As AccessViolationException
                        MsgBox("Error al crear la nueva carpeta, no tiene permisos para crear carpetas " & e.Message)
                    Catch e As ArgumentNullException
                        MsgBox("Error al crear la nueva carpeta, la dirección en la que crear la carpeta no puede contener partes null " & e.Message)
                    Catch e As FormatException
                        MsgBox("Error al crear la nueva carpeta, el formato de la dirección en la que crear la carpeta no es válido " & e.Message)
                    Catch e As DuplicateNameException
                        MsgBox("Error al crear la nueva carpeta, ya existe una carpeta con ese nombre " & e.Message)
                    End Try
                End If
            End If
            'Finalizar la impersonación
            oImp1.EndImpersonation()
        Catch err As ManagementException
            MessageBox.Show("An error occured while querying for WMI data: " _
                & err.Message)

        End Try
    End Sub

    Public Shared Sub deleteFolder(ByVal dir As String, ByVal server As String, ByVal nombre As String)
        '==========================================================================
        ' Name:     deleteFolder
        ' Purpose:  Eliminar una nueva carpeta en un recurso compartido de un equipo remmoto
        ' Input:    3 String: La dirección en la que se creará el directorio 'dir',
        '           El nombre de la nueva carpeta'nombre', El servidor 'server'
        ' Return:   None
        ' Calls:    None
        '==========================================================================
        Try
            Dim path As String
            'Crear la dirección donde se encuentra la carpeta a eliminar
            path = "\\" & server & "\" & dir & "\" & nombre

            If Directory.Exists(path) Then
                'Si la dirección es de un directorio se elimina la carpeta
                Directory.Delete(path, True)
            End If

        Catch err As ManagementException
            MessageBox.Show("An error occured while querying for WMI data: " _
                & err.Message)

        End Try
    End Sub

    Public Shared Sub deleteFolderArbol(ByVal dir As String)
        '==========================================================================
        ' Name:     deleteFolderArbol
        ' Purpose:  Eliminar una nueva carpeta en un recurso compartido de un equipo remmoto
        ' Input:    1 String: La dirección en la que se creará el directorio 'dir'
        ' Return:   None
        ' Calls:    None
        '==========================================================================
        Try
            Dim path As String
            path = dir

            'Impersonar para obtener permisos de controlador de dominio
            Dim oImp1 As Impersonation
            oImp1 = New Impersonation()
            Dim partes() As String = Settings.adminLogon.Split("\")
            oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)

            If Directory.Exists(path) Then
                'Si en la dirección hay un directorio, se elimina
                ' Evitar carpetas de sistema
                Dim objFSO, objFolder, colSubfolders, obj
                objFSO = CreateObject("Scripting.FileSystemObject")
                objFolder = objFSO.GetFolder(path)
                If Not (objFolder.Attributes > 21) Then
                    Try
                        Directory.Delete(path, True)
                    Catch e As AccessViolationException
                        MsgBox("Error al eliminar la nueva carpeta, no tiene permisos para eliminar carpetas " & e.Message)
                    Catch e As ArgumentNullException
                        MsgBox("Error al eliminar la nueva carpeta, la dirección de la carpeta no puede contener partes null " & e.Message)
                    Catch e As FormatException
                        MsgBox("Error al eliminar la nueva carpeta, el formato de la dirección de la carpeta no es válido " & e.Message)
                    End Try
                Else
                    MsgBox("No se pueden modificar las carpertas de sistema")
                End If
            End If
            'Fin de la impersonación
            oImp1.EndImpersonation()
        Catch err As ManagementException
            MessageBox.Show("An error occured while querying for WMI data: " _
                & err.Message)

        End Try
    End Sub

    Public Shared Sub renameFolder(ByVal dir As String, ByVal nuevo As String)
        '==========================================================================
        ' Name:     renameFolder
        ' Purpose:  Renombrar una nueva carpeta en un recurso compartido de un equipo remmoto
        ' Input:    2 String: La dirección en la que se creará el directorio 'dir', El nuevo nombre para la carpeta 'nuevo'
        ' Return:   None
        ' Calls:    None
        '==========================================================================
        Try
            'Impersonar para obtener permisos de controlador de dominio
            Dim oImp1 As Impersonation
            oImp1 = New Impersonation()
            Dim partes() As String = Settings.adminLogon.Split("\")
            oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)

            Dim path As String
            Dim parts() As String
            parts = dir.Split("\")

            If Not parts.Length > 2 Or isDirectory("\\" & parts(0) & "\" & parts(parts.Length - 1)) Then
                MsgBox("No se pueden renombrar ni los servidores ni los recursos compartidos")
            Else
                Dim objFSO, objFolder, colSubfolders, obj
                objFSO = CreateObject("Scripting.FileSystemObject")
                objFolder = objFSO.GetFolder("\\" & dir)
                If Not (objFolder.Attributes > 21) Then
                    'Si la dirección no da a un servidor o recurso compartido, se crea la nueva dirección
                    Dim nuevadir As String = ""
                    For i = 0 To parts.Length - 2
                        nuevadir = nuevadir & "\" & parts(i)
                    Next
                    'Se establece como la dirección de la carpeta
                    path = nuevadir
                    Try
                        Directory.Move("\\" & dir, "\" & path & "\" & nuevo)
                    Catch e As AccessViolationException
                        MsgBox("Error al renombrar la nueva carpeta, no tiene permisos para renombrar carpetas " & e.Message)
                    Catch e As ArgumentNullException
                        MsgBox("Error al renombrar la nueva carpeta, la dirección de la carpeta a renombrar no puede contener partes null " & e.Message)
                    Catch e As FormatException
                        MsgBox("Error al renombrar la nueva carpeta, el formato de la dirección de la carpeta a renombrar no es válido " & e.Message)
                    Catch e As DuplicateNameException
                        MsgBox("Error al renombrar la nueva carpeta, ya existe una carpeta con ese nombre " & e.Message)
                    End Try
                Else
                    MsgBox("No se pueden modificar las carpertas de sistema")
                End If
            End If
            oImp1.EndImpersonation()
        Catch err As ManagementException
            MessageBox.Show("An error occured while querying for WMI data: " _
                & err.Message)

        End Try
    End Sub

    Public Shared Function editableFolder(ByVal dir As String) As Boolean
        '==========================================================================
        ' Name:     renameFolder
        ' Purpose:  Renombrar una nueva carpeta en un recurso compartido de un equipo remmoto
        ' Input:    2 String: La dirección en la que se creará el directorio 'dir', El nuevo nombre para la carpeta 'nuevo'
        ' Return:   None
        ' Calls:    None
        '==========================================================================
        Try
            'Impersonar para obtener permisos de controlador de dominio
            Dim oImp1 As Impersonation
            oImp1 = New Impersonation()
            Dim partes() As String = Settings.adminLogon.Split("\")
            oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)

            Dim puede As Boolean = True
            Dim path As String
            Dim parts() As String
            parts = dir.Split("\")


            Dim objFSO, objFolder, colSubfolders, obj
            objFSO = CreateObject("Scripting.FileSystemObject")
            objFolder = objFSO.GetFolder(dir)
            If (objFolder.Attributes > 21) Then
                puede = False
            End If
            oImp1.EndImpersonation()
            Return puede
        Catch err As ManagementException
            MessageBox.Show("An error occured while querying for WMI data: " _
                & err.Message)

        End Try

    End Function

    Public Shared Sub renameProfile(ByVal equipo As String, ByVal viejo As String, ByVal nuevo As String)
        '==========================================================================
        ' Name:     renameProfile
        ' Purpose:  Renombrar un perfil en un recurso compartido de un equipo remmoto
        ' Input:    2 String: El nombre del equipo en el que está el perfil 'dir',
        '           El nuevo nombre para el perfíl 'nuevo', El nombre actual del perfíl
        ' Return:   None
        ' Calls:    None
        '==========================================================================
        Try
            'Impersonar para obtener permisos de controlador de dominio
            Dim oImp1 As Impersonation
            oImp1 = New Impersonation()
            Dim partes() As String = Settings.adminLogon.Split("\")
            oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)

            'Se crean las dos direcciones
            Dim viejoDir As String = "\\" & equipo & "\" & "C$\Documents and Settings\" & viejo
            Dim nuevoDir As String = "\\" & equipo & "\" & "C$\Documents and Settings\" & nuevo
            'Se establece la nueva dirección para el perfíl
            'Esto cambiará su nombre
            Try
                Directory.Move(viejoDir, nuevoDir)
            Catch e As AccessViolationException
                MsgBox("Error al renombrar el perfíl, no tiene permisos para renombrar el perfíl " & e.Message)
            Catch e As ArgumentNullException
                MsgBox("Error al renombrar el perfíl, la dirección o el nombre a con el que renombrar el perfíl es nulo " & e.Message)
            Catch e As FormatException
                MsgBox("Error al renombrar el perfíl, el formato de la dirección o del nuevo nombre no es válida " & e.Message)
            Catch e As DuplicateNameException
                MsgBox("Error al renombrar el perfíl, ya existe un perfíl con ese nombre " & e.Message)
            End Try

        Catch err As ManagementException
            MessageBox.Show("An error occured while querying for WMI data: " _
                & err.Message)

        End Try
    End Sub

    Public Shared Function isDirectory(ByVal dir As String) As Boolean
        '==========================================================================
        ' Name:     isDirectory
        ' Purpose:  Saber si una dirección apunta a un directorio
        ' Input:    1 String: La direccion del supuesto directorio 'dir'
        ' Return:   1 boolean: True si es un directorio, false en caso contrario
        ' Calls:    None
        '==========================================================================
        Try
            'Impersonar para obtener permisos de controlador de dominio
            Dim oImp1 As Impersonation
            oImp1 = New Impersonation()
            Dim partes() As String = Settings.adminLogon.Split("\")
            oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)

            Dim path As String

            path = dir
            'Si es un directorio 
            If Directory.Exists(path) Then
                oImp1.EndImpersonation()
                Return True
                'Si es un archivo
                'ElseIf File.Exists(path) Then
                '    oImp1.EndImpersonation()
                '    Return False
            Else
                oImp1.EndImpersonation()
                'Si no es ni una cosa ni la otra
                Return False
            End If


        Catch err As ManagementException
            MessageBox.Show("An error occured while querying for WMI data: " _
                & err.Message)

        End Try
    End Function

    Public Shared Function isProfile(ByVal computer As String, ByVal name As String) As Boolean
        '==========================================================================
        ' Name:     isDirectory
        ' Purpose:  Saber si existe un perfíl en un equipo con un determinado nombre 
        ' Input:    2 String: El equipo 'computer', El nombre del perfíl'name'
        ' Return:   1 boolean: True si es un directorio, false en caso contrario
        ' Calls:    None
        '==========================================================================
        Try

            'Impersonar para obtener permisos de controlador de dominio
            Dim oImp1 As Impersonation
            oImp1 = New Impersonation()
            Dim partes() As String = Settings.adminLogon.Split("\")
            oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)

            Dim path As String
            'Se crea la dirección donde buscar
            path = "\\" & computer & "\" & "C$\Documents and Settings\" & name
            'Si la dirección apunta a un directorio
            If Directory.Exists(path) Then

                Return True
                'Si apunta a un archivo
            ElseIf File.Exists(path) Then
                Return False
            Else
                'Si no apunta a una cosa ni la otra
                Return False
            End If

        Catch err As ManagementException
            MessageBox.Show("An error occured while querying for WMI data: " _
                & err.Message)

        End Try
    End Function

    Public Shared Function SFolderelements3(ByVal direccion As String) As List(Of String)
        '==========================================================================
        ' Name:     SFolderelements3
        ' Purpose:  Obtener la lista de los nombres de los elementos que contiene un directorio 
        ' Input:    1 String: La dirección de la carpeta 'direccion'
        ' Return:   1 array: Array de strings con los nombres de los elementos
        ' Calls:    None
        '==========================================================================
        Try

            Dim path As String
            Dim elements As New List(Of String)
            'Impersonar para obtener permisos de controlador de dominio
            Dim oImp1 As Impersonation
            oImp1 = New Impersonation()
            Dim partes() As String = Settings.adminLogon.Split("\")
            oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)
            'Dim WIdentity As WindowsIdentity
            'Dim logonToken As IntPtr = WIdentity.GetCurrent().Token
            'WIdentity.Impersonate(logonToken)

            path = direccion.Substring(2)
            Dim Parts() As String = path.Split("\")

            Dim computer As String = Parts(0)

            Dim folder As String = Parts(Parts.Length - 1)

            'Si la direcciíon apunta a un directorio
            'If Directory.Exists(direccion) Then

            ''Si no apunta a una carpeta de sistema
            'If (Not folder.Contains(".LOG")) And (Not folder.Contains("REMOVE")) Then
            Dim objFSO, objFolder, colSubfolders, obj
            objFSO = CreateObject("Scripting.FileSystemObject")
            'Dim dspecial = objFSO.GetSpecialFolder(direccion)

            'Si no apunta a una carpeta de sistema
            'If dspecial Is Nothing Then
            objFolder = objFSO.GetFolder(direccion)

            If Not (objFolder.Attributes > 21) Then
                

                'Obtener archivos
                colSubfolders = objFolder.Subfolders
                If colSubfolders.count > 0 Then

                    'Incluirlos en el array
                    For Each obj In colSubfolders

                        Dim carpeta As String = obj.Name
                        elements.Add(carpeta)
                    Next obj
                End If


                'colSubfolders = objFolder.subfolders
                ''Obtener carpetas
                'For Each obj In colSubfolders
                '    'Incluirlas en el array
                '    Dim carpeta As String = obj.Name
                '    elements.Add(carpeta)

                'Next obj
            End If
            oImp1.EndImpersonation()
            Return elements

            'End If

        Catch err As ManagementException
            MessageBox.Show("An error occured while querying for WMI data: " _
                & err.Message)

        End Try
    End Function

    Public Shared Function IExplorerDatos(ByVal computer As String) As IList(Of String)
        '==========================================================================
        ' Name:     IExplorerDatos
        ' Purpose:  Obtener los datos de conexión de InternetExplorer 
        ' Input:    1 String: El nombre del equipo 'computer'
        ' Return:   1 array: Array de strings con los valores de los elementos
        ' Calls:    None
        '==========================================================================
        Dim resultados As New List(Of String)

        Const HKEY_CURRENT_USER = &H80000001
        Try

            'Crear el objeto del que obtener los datos, obteniendo permisos de controlador de dominio
            Dim oLocator As Object = CreateObject("WbemScripting.SWbemLocator")
            Dim strComputer As String = computer

            Dim oService As Object = oLocator.ConnectServer(strComputer, "Root\DEFAULT", Settings.adminLogon, Settings.adminPassword)

            'oService.Security_.impersonationlevel = 3
            Dim objRegistry As Object = oService.Get("StdRegProv")

            'objRegistry = GetObject("winmgmts:{impersonationLevel=imper sonate}!\\" & strComputer & "\root\default:StdRegProv")

            'Dim strKeyPath As String = "SOFTWARE\Microsoft\\Windows\CurrentVersion\Internet Settings"
            Dim strKeyPath As String = "SOFTWARE\Microsoft\Windows\CurrentVersion\Internet Settings"

            'Obtener dato ProxyEnable e insertarlo en el array
            Dim strValueName As String = "ProxyEnable"
            Dim dwVal As String
            Try
                objRegistry.GetDWORDValue(HKEY_CURRENT_USER, strKeyPath, strValueName, dwVal)
                resultados.Add(dwVal)
            Catch err As Exception
                MessageBox.Show("Error al cargar los datos de Intenet Explorer del equipo seleccionado: " _
                    & err.Message)
            End Try
            'Obtener dato ProxyServer e insertarlo en el array
            strValueName = "ProxyServer"
            Dim strVal As String
            Try
                objRegistry.GetStringValue(HKEY_CURRENT_USER, strKeyPath, strValueName, strVal)
                resultados.Add(strVal)

            Catch e As InvalidCastException
                resultados.Add("")
            Catch err As Exception
                MessageBox.Show("Error al cargar los datos de Intenet Explorer del equipo seleccionado, error al cargar el nombre de proxy: " _
                    & err.Message)
            End Try

            'Obtener dato ProxyOverride e insertarlo en el array
            strValueName = "ProxyOverride"
            Dim strV As String
            Try
                objRegistry.GetStringValue(HKEY_CURRENT_USER, strKeyPath, strValueName, strV)
                resultados.Add(strV)
            Catch e As InvalidCastException
                resultados.Add("")
            Catch err As Exception
                MessageBox.Show("Error al cargar los datos de Intenet Explorer del equipo seleccionado, error al cargar las excepciones de proxy: " _
                    & err.Message)
            End Try

            Return resultados

        Catch err As Exception
            MessageBox.Show("El nombre que se ha insertado para cargar los datos de internet explorer no es válido: " _
                & err.Message)
        End Try
    End Function

    Public Shared Sub IExplorerCarga(ByVal computer As String, ByVal enabled As String, ByVal proxy As String, ByVal exceptions As String)
        '==========================================================================
        ' Name:     IExplorerDatos
        ' Purpose:  Cargar los datos de conexión de InternetExplorer 
        ' Input:    3 String: El nombre del equipo 'computer', El valor habilitadodel IExplorer'enabled',
        '           El valor para proxyServer 'proxy', Las excepciones para ProxyOverride 'Excepctions'
        ' Return:   None
        ' Calls:    None
        '==========================================================================

        Const HKEY_CURRENT_USER = &H80000001
        Try
            'Crear el objeto del que obtener los datos, obteniendo permisos de controlador de dominio
            Dim oLocator As Object = CreateObject("WbemScripting.SWbemLocator")
            Dim strComputer As String = computer
            Dim oService As Object = oLocator.ConnectServer(strComputer, "Root\DEFAULT", Settings.adminLogon, Settings.adminPassword)
            Dim objRegistry As Object = oService.Get("StdRegProv")
            Dim strKeyPath As String = "SOFTWARE\Microsoft\\Windows\CurrentVersion\Internet Settings"

            'Cargar dato ProxyEnable 
            Dim strValueName As String = "ProxyEnable"
            Dim dwValue As String
            If enabled = 1 Then
                dwValue = 1
            Else
                dwValue = 0
            End If
            Try
                objRegistry.SetDWORDValue(HKEY_CURRENT_USER, strKeyPath, strValueName, dwValue)
            Catch e As AccessViolationException
                MsgBox("Error al establecer el estado(habilitado/deshabilitado) de Internet Explorer, no tiene permisos para establecer el estado de internet explorer " & e.Message)
            Catch e As ArgumentNullException
                MsgBox("Error al establecer el estado(habilitado/deshabilitado) de Internet Explorer, no puede haber argumentos nulos " & e.Message)
            Catch e As FormatException
                MsgBox("Error al establecer el estado(habilitado/deshabilitado) de Internet Explorer, uno o varios argumentos no están en el formato adecuado " & e.Message)
            End Try
            'Cargar dato ProxyServer
            strValueName = "ProxyServer"
            Dim strVal As String = proxy
            Try
                objRegistry.SetStringValue(HKEY_CURRENT_USER, strKeyPath, strValueName, strVal)
            Catch e As AccessViolationException
                MsgBox("Error al establecer el proxy para Internet Explorer, no tiene permisos para establecer el proxy para internet explorer " & e.Message)
            Catch e As ArgumentNullException
                MsgBox("Error al establecer el proxy para Internet Explorer, no puede haber argumentos nulos " & e.Message)
            Catch e As FormatException
                MsgBox("Error al establecer el proxy para Internet Explorer uno o varios argumentos no están en el formato adecuado " & e.Message)
            End Try
            'Cargar dato ProxyOverride
            strValueName = "ProxyOverride"
            Dim strValue As String
            strValue = exceptions
            Try
                objRegistry.SetStringValue(HKEY_CURRENT_USER, strKeyPath, strValueName, strValue)
            Catch e As AccessViolationException
                MsgBox("Error al establecer las excepciones para Internet Explorer, no tiene permisos para establecer el proxy para internet explorer " & e.Message)
            Catch e As ArgumentNullException
                MsgBox("Error al establecer las excepciones para Internet Explorer, no puede haber argumentos nulos " & e.Message)
            Catch e As FormatException
                MsgBox("Error al establecer las excepciones para Internet Explorer uno o varios argumentos no están en el formato adecuado " & e.Message)
            End Try
        Catch err As Exception

            MessageBox.Show("Nombre de máquina no válido: " _
                & err.Message)

        End Try
    End Sub

    '-------------------------------------------Seguridad carpetas----------------------------------------------
    Public Shared Function getElementsP(ByVal carpeta As String) As List(Of String)
        '==========================================================================
        ' Name:     getSharedElements
        ' Purpose:  Obtener una lista con los nombres de los usuarios o grupos que poseen permisos sobre la carpeta
        ' Input:    1 String: El nombre del servidor 'server'
        ' Return:   1 array: array de strings en los que cada elemento es el nombre de un recurso compartido
        ' Calls:    None
        '==========================================================================
        'Impersonar para obtener permisos de controlador de dominio
        'Impersonar para obtener permisos de controlador de dominio
        Dim oImp1 As Impersonation
        oImp1 = New Impersonation()
        Dim partes() As String = Settings.adminLogon.Split("\")
        oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)

        Dim lNew As New List(Of String)
        Try
            'Obtner la lista de permisos sobre la carpeta
            Dim myinfo As New DirectoryInfo(carpeta)
            Dim dSecurity As DirectorySecurity = myinfo.GetAccessControl()
            Dim fSecurity As AuthorizationRuleCollection = _
            dSecurity.GetAccessRules(True, True,
            Type.GetType("System.Security.Principal.NTAccount"))

            For Each myacc As Security.AccessControl.AccessRule In fSecurity
                'Casting para pasar a permisos del tipo archivo
                Dim acc As FileSystemAccessRule = DirectCast(myacc, 
                FileSystemAccessRule)

                'Añadir a la lista los nombres de los objetos a los que se han concedido los permisos 
                If Not lNew.Contains(acc.IdentityReference.Value) Then
                    lNew.Add(acc.IdentityReference.Value)
                End If

            Next
        Catch e As FormatException
            MsgBox("Error de formato al obtener los permisos sobre " & carpeta & e.Message)
        Catch e As AccessViolationException
            MsgBox("Error al obtener los permisos, No tienes permisos para acceder a este dato " & e.Message)
        Catch e As ArgumentOutOfRangeException
            MsgBox("Error al obtener los permisos,Se ha intentado obtener un dato con un índice fuera de rango en el large integer " & e.Message)
        Catch e As OperationCanceledException
            MsgBox("Error al obtener los permisos,Se está intentando obtener un dato involucrado en una operación cancelada en el large integer " & e.Message)
        Catch e As OperationAbortedException
            MsgBox("Error al obtener los permisos,Se está intentando obtener un dato involucrado en una operación abortada en el large integer " & e.Message)
        Catch e As OutOfMemoryException
            MsgBox("Error al obtener los permisos,La memoria está totalmente ocupada " & e.Message)
        Catch e As FieldAccessException
            MsgBox("Error al obtener los permisos,  acceder a al ubicación física del dato en el large integer " & e.Message)
        Catch e As InsufficientMemoryException
            MsgBox("Error al obtener los permisos,La memoria está totalmente ocupada " & e.Message)
        Catch e As MissingFieldException
            MsgBox("Error al obtener los permisos, al acceder a al ubicación física del dato, no existe " & e.Message)
        End Try
        oImp1.EndImpersonation()
        Return (lNew)


    End Function

    Public Shared Function getElementsPH(ByVal carpeta As String) As List(Of String)
        '==========================================================================
        ' Name:     getSharedElements
        ' Purpose:  Obtener una lista con los nombres de los usuarios o grupos que poseen permisos sobre la carpeta
        ' Input:    1 String: El nombre del servidor 'server'
        ' Return:   1 array: array de strings en los que cada elemento es el nombre de un recurso compartido
        ' Calls:    None
        '==========================================================================
        'Impersonar para obtener permisos de controlador de dominio
        'Impersonar para obtener permisos de controlador de dominio
        Dim oImp1 As Impersonation
        oImp1 = New Impersonation()
        Dim partes() As String = Settings.adminLogon.Split("\")
        oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)

        Dim lNew As New List(Of String)
        Try
            'Obtner la lista de permisos sobre la carpeta
            Dim myinfo As New DirectoryInfo(carpeta)
            Dim dSecurity As DirectorySecurity = myinfo.GetAccessControl()
            Dim fSecurity As AuthorizationRuleCollection = _
            dSecurity.GetAccessRules(True, True,
            Type.GetType("System.Security.Principal.NTAccount"))

            For Each myacc As Security.AccessControl.AccessRule In fSecurity
                'Casting para pasar a permisos del tipo archivo
                Dim acc As FileSystemAccessRule = DirectCast(myacc, 
                FileSystemAccessRule)

                'Añadir a la lista los nombres de los objetos a los que se han concedido los permisos 
                If Not lNew.Contains(acc.IdentityReference.Value) Then
                    If acc.IsInherited Then
                        'Si se ha heredado 
                        lNew.Add(acc.IdentityReference.Value)
                    End If

                End If

            Next
        Catch e As FormatException
            MsgBox("Error de formato al obtener los permisos sobre " & carpeta & e.Message)
        Catch e As AccessViolationException
            MsgBox("Error al obtener los permisos, No tienes permisos para acceder a este dato " & e.Message)
        Catch e As ArgumentOutOfRangeException
            MsgBox("Error al obtener los permisos,Se ha intentado obtener un dato con un índice fuera de rango en el large integer " & e.Message)
        Catch e As OperationCanceledException
            MsgBox("Error al obtener los permisos,Se está intentando obtener un dato involucrado en una operación cancelada en el large integer " & e.Message)
        Catch e As OperationAbortedException
            MsgBox("Error al obtener los permisos,Se está intentando obtener un dato involucrado en una operación abortada en el large integer " & e.Message)
        Catch e As OutOfMemoryException
            MsgBox("Error al obtener los permisos,La memoria está totalmente ocupada " & e.Message)
        Catch e As FieldAccessException
            MsgBox("Error al obtener los permisos,  acceder a al ubicación física del dato en el large integer " & e.Message)
        Catch e As InsufficientMemoryException
            MsgBox("Error al obtener los permisos,La memoria está totalmente ocupada " & e.Message)
        Catch e As MissingFieldException
            MsgBox("Error al obtener los permisos, al acceder a al ubicación física del dato, no existe " & e.Message)
        End Try
        oImp1.EndImpersonation()
        Return (lNew)


    End Function

    Public Shared Function GetPermission(ByVal carpeta As String, ByVal user As String, ByVal permis As String) As Boolean()
        '==========================================================================
        ' Name:     getSharedElements
        ' Purpose:  Obtener una lista con los nombres de los usuarios o grupos que poseen permisos sobre la carpeta
        ' Input:    1 String: El nombre del servidor 'server'
        ' Return:   1 array: array de strings en los que cada elemento es el nombre de un recurso compartido
        ' Calls:    None
        '==========================================================================
        Dim resul(3) As Boolean
        resul(0) = False
        resul(1) = False
        resul(2) = False
        resul(3) = False

        'Impersonar para obtener permisos de controlador de dominio
        Dim oImp1 As Impersonation
        oImp1 = New Impersonation()
        Dim part() As String = Settings.adminLogon.Split("\")
        oImp1.StartImpersonation(part(1), Settings.userDNSDomain, Settings.adminPassword)

        'Dim lNew As New List(Of String)
        Try
            'Obtener los permisos concedidos sobre la carpeta
            Dim myinfo As New DirectoryInfo(carpeta)
            Dim dSecurity As DirectorySecurity = myinfo.GetAccessControl()
            Dim fSecurity As AuthorizationRuleCollection = _
            dSecurity.GetAccessRules(True, True,
            Type.GetType("System.Security.Principal.NTAccount"))

            For Each myacc As Security.AccessControl.AccessRule In fSecurity
                'Pasar el permiso al tipo permiso sobre archivo
                Dim acc As FileSystemAccessRule = DirectCast(myacc, 
                FileSystemAccessRule)

                'Buscar el permiso
                If Not acc.PropagationFlags = 2 Then
                    Dim partes() As String = acc.IdentityReference.Value.Split("\")
                    If partes.Last = user Then
                        'Para cada tipo de permiso...
                        If permis = "Control total" Then
                            If (acc.FileSystemRights And FileSystemRights.FullControl) = FileSystemRights.FullControl Then
                                'Si se ha encontrado
                                If acc.AccessControlType = AccessControlType.Allow Then
                                    'Si es del tipo permitido
                                    resul(0) = True
                                    If acc.IsInherited Then
                                        'Si se ha heredado 
                                        resul(2) = True
                                    End If
                                ElseIf acc.AccessControlType = AccessControlType.Deny Then
                                    'Si es del tipo denegado
                                    resul(1) = True
                                    If acc.IsInherited Then
                                        'Si se ha heredado
                                        resul(2) = True
                                    End If
                                End If
                            End If
                        ElseIf permis = "Modificar" Then
                            If (acc.FileSystemRights And FileSystemRights.Modify) = FileSystemRights.Modify Then
                                If acc.AccessControlType = AccessControlType.Allow Then
                                    resul(0) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                ElseIf acc.AccessControlType = AccessControlType.Deny Then
                                    resul(1) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                End If
                            End If
                        ElseIf permis = "Lectura y ejecución" Then
                            If (acc.FileSystemRights And FileSystemRights.ReadAndExecute) = FileSystemRights.ReadAndExecute Then
                                If acc.AccessControlType = AccessControlType.Allow Then
                                    resul(0) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                ElseIf acc.AccessControlType = AccessControlType.Deny Then
                                    resul(1) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                End If
                            End If
                        ElseIf permis = "Mostrar el contenido de la carpeta" Then
                            If (acc.FileSystemRights And FileSystemRights.ListDirectory) = FileSystemRights.ListDirectory Then
                                If acc.AccessControlType = AccessControlType.Allow Then
                                    resul(0) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                ElseIf acc.AccessControlType = AccessControlType.Deny Then
                                    resul(1) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                End If
                            End If
                        ElseIf permis = "Leer" Then
                            If (acc.FileSystemRights And FileSystemRights.Read) = FileSystemRights.Read Then
                                If acc.AccessControlType = AccessControlType.Allow Then
                                    resul(0) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                ElseIf acc.AccessControlType = AccessControlType.Deny Then
                                    resul(1) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                End If
                            End If
                        ElseIf permis = "Escribir" Then
                            If (acc.FileSystemRights And FileSystemRights.Write) = FileSystemRights.Write Then
                                If acc.AccessControlType = AccessControlType.Allow Then
                                    resul(0) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                ElseIf acc.AccessControlType = AccessControlType.Deny Then
                                    resul(1) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                End If
                            End If
                        End If
                    End If
                End If

            Next
        Catch e As FormatException
            MsgBox("Error de formato al obtener los permisos sobre " & carpeta & e.Message)
        Catch e As AccessViolationException
            MsgBox("Error al obtener los permisos, No tienes permisos para acceder a este dato " & e.Message)
        Catch e As ArgumentOutOfRangeException
            MsgBox("Error al obtener los permisos,Se ha intentado obtener un dato con un índice fuera de rango en el large integer " & e.Message)
        Catch e As OperationCanceledException
            MsgBox("Error al obtener los permisos,Se está intentando obtener un dato involucrado en una operación cancelada en el large integer " & e.Message)
        Catch e As OperationAbortedException
            MsgBox("Error al obtener los permisos,Se está intentando obtener un dato involucrado en una operación abortada en el large integer " & e.Message)
        Catch e As OutOfMemoryException
            MsgBox("Error al obtener los permisos,La memoria está totalmente ocupada " & e.Message)
        Catch e As FieldAccessException
            MsgBox("Error al obtener los permisos,  acceder a al ubicación física del dato en el large integer " & e.Message)
        Catch e As InsufficientMemoryException
            MsgBox("Error al obtener los permisos,La memoria está totalmente ocupada " & e.Message)
        Catch e As MissingFieldException
            MsgBox("Error al obtener los permisos, al acceder a al ubicación física del dato, no existe " & e.Message)
        End Try

        oImp1.EndImpersonation()
        Return resul
    End Function

    Public Shared Function AddPermission(ByVal carpeta As String, ByVal user As String, ByVal permis As String, ByVal tipo As String)
        '==========================================================================
        ' Name:     AddPermission
        ' Purpose:  Añadir a un objeto un permiso sobre la carpeta
        ' Input:    1 String: El nombre del servidor 'server'
        ' Return:   1 array: array de strings en los que cada elemento es el nombre de un recurso compartido
        ' Calls:    None
        '==========================================================================

        'Impersonar para obtener permisos de controlador de dominio
        Dim oImp1 As Impersonation
        oImp1 = New Impersonation()
        Dim partes() As String = Settings.adminLogon.Split("\")
        oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)

        'Obtener la lista de permisos sobre la carpeta
        Dim myinfo As New DirectoryInfo(carpeta)
        Dim dSecurity As DirectorySecurity = myinfo.GetAccessControl()
        'Dim fSecurity As AuthorizationRuleCollection = _
        'dSecurity.GetAccessRules(True, True,
        'Type.GetType("System.Security.Principal.NTAccount"))
        Try
            'Crear el permiso y añadirlo a la lista
            If permis = "Control total" Then
                If tipo = "allow" Then
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.FullControl, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Allow))
                Else
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.FullControl, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Deny))
                End If
            ElseIf permis = "Modificar" Then
                If tipo = "allow" Then
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.Modify, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Allow))
                Else
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.Modify, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Deny))
                End If
            ElseIf permis = "Lectura y ejecución" Then
                If tipo = "allow" Then
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.ReadAndExecute, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Allow))
                Else
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.ReadAndExecute, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Deny))
                End If
            ElseIf permis = "Mostrar el contenido de la carpeta" Then
                If tipo = "allow" Then
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.ListDirectory, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Allow))
                Else
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.ListDirectory, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Deny))
                End If
            ElseIf permis = "Leer" Then
                If tipo = "allow" Then
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.Read, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Allow))
                Else
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.Read, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Deny))
                End If
            ElseIf permis = "Escribir" Then
                If tipo = "allow" Then
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.Write, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Allow))
                Else
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.Write, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Deny))
                End If
            End If
        Catch e As AccessViolationException
            MsgBox("Error al crear un permiso sobre la carpeta " & e.Message)
        Catch e As ConstraintException
            MsgBox("Error, al crear un permiso sobre la carpeta se viola una restricción " & e.Message)
        End Try

        Try
            'Asignar la nueva lista de permisos a la carpeta
            myinfo.SetAccessControl(dSecurity)
        Catch e As AccessViolationException
            MsgBox("No se poseen permisos para añadir permisos a la carpeta " & e.Message)
        Catch e As ConstraintException
            MsgBox("Error, Al conceder el permiso sobre la carpeta se viola una restricción " & e.Message)
        End Try
        oImp1.EndImpersonation()

    End Function

    Public Shared Function RemovePermission(ByVal carpeta As String, ByVal user As String, ByVal permis As String, ByVal tipo As String)
        '==========================================================================
        ' Name:     AddPermission
        ' Purpose:  Quitar a un objeto un permiso sobre la carpeta
        ' Input:    1 String: El nombre del servidor 'server'
        ' Return:   1 array: array de strings en los que cada elemento es el nombre de un recurso compartido
        ' Calls:    None
        '==========================================================================

        'Impersonar para obtener permisos de controlador de dominio
        Dim oImp1 As Impersonation
        oImp1 = New Impersonation()
        Dim partes() As String = Settings.adminLogon.Split("\")
        oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)

        'Obtener la lista de permisos sobre la carpeta
        Dim myinfo As New DirectoryInfo(carpeta)
        Dim dSecurity As DirectorySecurity = myinfo.GetAccessControl()
        'Dim fSecurity As AuthorizationRuleCollection = _
        'dSecurity.GetAccessRules(True, True,
        'Type.GetType("System.Security.Principal.NTAccount"))

        Try
            'Eliminar el permiso, buscándolo por su tipo
            If permis = "Control total" Then
                If tipo = "allow" Then
                    dSecurity.RemoveAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.FullControl, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Allow))
                Else
                    dSecurity.RemoveAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.FullControl, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Deny))
                End If
            ElseIf permis = "Modificar" Then
                If tipo = "allow" Then
                    dSecurity.RemoveAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.Modify, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Allow))
                Else
                    dSecurity.RemoveAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.Modify, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Deny))
                End If
            ElseIf permis = "Lectura y ejecución" Then
                If tipo = "allow" Then
                    dSecurity.RemoveAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.ReadAndExecute, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Allow))
                Else
                    dSecurity.RemoveAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.ReadAndExecute, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Deny))
                End If
            ElseIf permis = "Mostrar el contenido de la carpeta" Then
                If tipo = "allow" Then
                    dSecurity.RemoveAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.ListDirectory, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Allow))
                Else
                    dSecurity.RemoveAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.ListDirectory, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Deny))
                End If
            ElseIf permis = "Leer" Then
                If tipo = "allow" Then
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.Read, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Allow))
                Else
                    dSecurity.AddAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.Read, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Deny))
                End If
            ElseIf permis = "Escribir" Then
                If tipo = "allow" Then
                    dSecurity.RemoveAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.Write, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Allow))
                Else
                    dSecurity.RemoveAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), FileSystemRights.Write, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, AccessControlType.Deny))
                End If
            End If
        Catch e As AccessViolationException
            MsgBox("Error al eliminar un permiso de la lista de permisos sobre la carpeta, se viola una restricción " & e.Message)
        Catch e As ConstraintException
            MsgBox("Error, al eliminar un permiso de la lista de permisos sobre la carpeta, error de formato " & e.Message)
        End Try
        Try
            'Asignar la nueva lista de permisos a la carpeta
            myinfo.SetAccessControl(dSecurity)
        Catch e As AccessViolationException
            MsgBox("No se poseen permisos para añadir permisos a la carpeta " & e.Message)
        Catch e As ConstraintException
            MsgBox("Error, Al conceder el permiso sobre la carpeta se viola una restricción " & e.Message)
        End Try
        oImp1.EndImpersonation()

    End Function

    Public Shared Function RemoveAllPermissions(ByVal carpeta As String, ByVal user As String)
        '==========================================================================
        ' Name:     AddPermission
        ' Purpose:  Quitar a un objeto un permiso sobre la carpeta
        ' Input:    1 String: El nombre del servidor 'server'
        ' Return:   1 array: array de strings en los que cada elemento es el nombre de un recurso compartido
        ' Calls:    None
        '==========================================================================

        'Impersonar para obtener permisos de controlador de dominio
        Dim oImp1 As Impersonation
        oImp1 = New Impersonation()
        Dim partes() As String = Settings.adminLogon.Split("\")
        oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)

        'Obtener la lista de permisos sobre la carpeta
        Dim myinfo As New DirectoryInfo(carpeta)
        Dim dSecurity As DirectorySecurity = myinfo.GetAccessControl()
        Dim fSecurity As AuthorizationRuleCollection = _
        dSecurity.GetAccessRules(True, True,
        Type.GetType("System.Security.Principal.NTAccount"))
        Try
            'Para cada permiso
            For Each myacc As Security.AccessControl.AccessRule In fSecurity
                Dim acc As FileSystemAccessRule = DirectCast(myacc, 
                FileSystemAccessRule)

                'Eliminarlo si pertenece al objeto al que se le quieren quitar todos los permisos
                dSecurity.RemoveAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), acc.FileSystemRights, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, acc.AccessControlType))

            Next
        Catch e As AccessViolationException
            MsgBox("Error al obtener la lista de permisos, se viola una restricción " & e.Message)
        Catch e As ConstraintException
            MsgBox("Error al obtener la lista de permisos, error de formato " & e.Message)
        End Try

        Try
            myinfo.SetAccessControl(dSecurity)
        Catch e As AccessViolationException
            MsgBox("No se poseen permisos para añadir permisos a la carpeta " & e.Message)
        Catch e As ConstraintException
            MsgBox("Error, Al conceder el permiso sobre la carpeta se viola una restricción " & e.Message)
        End Try
        oImp1.EndImpersonation()

    End Function

    Public Shared Sub RemoveHPerms(ByVal carpeta As String)
        '==========================================================================
        ' Name:     getSharedElements
        ' Purpose:  Obtener una lista con los nombres de los usuarios o grupos que poseen permisos sobre la carpeta
        ' Input:    1 String: El nombre del servidor 'server'
        ' Return:   1 array: array de strings en los que cada elemento es el nombre de un recurso compartido
        ' Calls:    None
        '==========================================================================
        'Impersonar para obtener permisos de controlador de dominio
        'Impersonar para obtener permisos de controlador de dominio
        Dim oImp1 As Impersonation
        oImp1 = New Impersonation()
        Dim partes() As String = Settings.adminLogon.Split("\")
        oImp1.StartImpersonation(partes(1), Settings.userDNSDomain, Settings.adminPassword)


        Try
            'Obtner la lista de permisos sobre la carpeta
            Dim myinfo As New DirectoryInfo(carpeta)
            Dim dSecurity As DirectorySecurity = myinfo.GetAccessControl()
            Dim fSecurity As AuthorizationRuleCollection = _
            dSecurity.GetAccessRules(True, True,
            Type.GetType("System.Security.Principal.NTAccount"))

            For Each myacc As Security.AccessControl.AccessRule In fSecurity
                'Casting para pasar a permisos del tipo archivo
                Dim acc As FileSystemAccessRule = DirectCast(myacc, 
                FileSystemAccessRule)

                'Añadir a la lista los nombres de los objetos a los que se han concedido los permisos 

                If acc.IsInherited Then
                    'Si se ha heredado 
                    'dSecurity.RemoveAccessRule(New FileSystemAccessRule(MailClass.GetTrustee(user), acc.FileSystemRights, InheritanceFlags.ContainerInherit Or InheritanceFlags.ObjectInherit, PropagationFlags.None, acc.AccessControlType))
                    dSecurity.RemoveAccessRule(acc)
                End If

            Next
        Catch e As FormatException
            MsgBox("Error de formato al obtener los permisos sobre " & carpeta & e.Message)
        Catch e As AccessViolationException
            MsgBox("Error al obtener los permisos, No tienes permisos para acceder a este dato " & e.Message)
        Catch e As ArgumentOutOfRangeException
            MsgBox("Error al obtener los permisos,Se ha intentado obtener un dato con un índice fuera de rango en el large integer " & e.Message)
        Catch e As OperationCanceledException
            MsgBox("Error al obtener los permisos,Se está intentando obtener un dato involucrado en una operación cancelada en el large integer " & e.Message)
        Catch e As OperationAbortedException
            MsgBox("Error al obtener los permisos,Se está intentando obtener un dato involucrado en una operación abortada en el large integer " & e.Message)
        Catch e As OutOfMemoryException
            MsgBox("Error al obtener los permisos,La memoria está totalmente ocupada " & e.Message)
        Catch e As FieldAccessException
            MsgBox("Error al obtener los permisos,  acceder a al ubicación física del dato en el large integer " & e.Message)
        Catch e As InsufficientMemoryException
            MsgBox("Error al obtener los permisos,La memoria está totalmente ocupada " & e.Message)
        Catch e As MissingFieldException
            MsgBox("Error al obtener los permisos, al acceder a al ubicación física del dato, no existe " & e.Message)
        End Try
        oImp1.EndImpersonation()



    End Sub

    Public Shared Function GetInHeritedPermission(ByVal carpeta As String, ByVal user As String, ByVal permis As String) As Boolean()
        '==========================================================================
        ' Name:     getSharedElements
        ' Purpose:  Obtener una lista con los nombres de los usuarios o grupos que poseen permisos sobre la carpeta
        ' Input:    1 String: El nombre del servidor 'server'
        ' Return:   1 array: array de strings en los que cada elemento es el nombre de un recurso compartido
        ' Calls:    None
        '==========================================================================
        Dim resul(3) As Boolean
        resul(0) = False
        resul(1) = False
        resul(2) = False
        resul(3) = False

        'Impersonar para obtener permisos de controlador de dominio
        Dim oImp1 As Impersonation
        oImp1 = New Impersonation()
        Dim part() As String = Settings.adminLogon.Split("\")
        oImp1.StartImpersonation(part(1), Settings.userDNSDomain, Settings.adminPassword)

        'Dim lNew As New List(Of String)
        Try
            'Obtener los permisos concedidos sobre la carpeta
            Dim myinfo As New DirectoryInfo(carpeta)
            Dim dSecurity As DirectorySecurity = myinfo.GetAccessControl()
            Dim fSecurity As AuthorizationRuleCollection = _
            dSecurity.GetAccessRules(True, True,
            Type.GetType("System.Security.Principal.NTAccount"))

            For Each myacc As Security.AccessControl.AccessRule In fSecurity
                'Pasar el permiso al tipo permiso sobre archivo
                Dim acc As FileSystemAccessRule = DirectCast(myacc, 
                FileSystemAccessRule)

                If (acc.InheritanceFlags And InheritanceFlags.ContainerInherit) Or (acc.InheritanceFlags And InheritanceFlags.ObjectInherit) Then

                End If
                'Buscar el permiso
                If Not acc.PropagationFlags = 2 Then
                    Dim partes() As String = acc.IdentityReference.Value.Split("\")
                    If partes.Last = user Then
                        'Para cada tipo de permiso...
                        If permis = "Control total" Then
                            If (acc.FileSystemRights And FileSystemRights.FullControl) = FileSystemRights.FullControl Then
                                'Si se ha encontrado
                                If acc.AccessControlType = AccessControlType.Allow Then
                                    'Si es del tipo permitido
                                    resul(0) = True
                                    If acc.IsInherited Then
                                        'Si se ha heredado 
                                        resul(2) = True
                                    End If
                                ElseIf acc.AccessControlType = AccessControlType.Deny Then
                                    'Si es del tipo denegado
                                    resul(1) = True
                                    If acc.IsInherited Then
                                        'Si se ha heredado
                                        resul(2) = True
                                    End If
                                End If
                            End If
                        ElseIf permis = "Modificar" Then
                            If (acc.FileSystemRights And FileSystemRights.Modify) = FileSystemRights.Modify Then
                                If acc.AccessControlType = AccessControlType.Allow Then
                                    resul(0) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                ElseIf acc.AccessControlType = AccessControlType.Deny Then
                                    resul(1) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                End If
                            End If
                        ElseIf permis = "Lectura y ejecución" Then
                            If (acc.FileSystemRights And FileSystemRights.ReadAndExecute) = FileSystemRights.ReadAndExecute Then
                                If acc.AccessControlType = AccessControlType.Allow Then
                                    resul(0) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                ElseIf acc.AccessControlType = AccessControlType.Deny Then
                                    resul(1) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                End If
                            End If
                        ElseIf permis = "Mostrar el contenido de la carpeta" Then
                            If (acc.FileSystemRights And FileSystemRights.ListDirectory) = FileSystemRights.ListDirectory Then
                                If acc.AccessControlType = AccessControlType.Allow Then
                                    resul(0) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                ElseIf acc.AccessControlType = AccessControlType.Deny Then
                                    resul(1) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                End If
                            End If
                        ElseIf permis = "Leer" Then
                            If (acc.FileSystemRights And FileSystemRights.Read) = FileSystemRights.Read Then
                                If acc.AccessControlType = AccessControlType.Allow Then
                                    resul(0) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                ElseIf acc.AccessControlType = AccessControlType.Deny Then
                                    resul(1) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                End If
                            End If
                        ElseIf permis = "Escribir" Then
                            If (acc.FileSystemRights And FileSystemRights.Write) = FileSystemRights.Write Then
                                If acc.AccessControlType = AccessControlType.Allow Then
                                    resul(0) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                ElseIf acc.AccessControlType = AccessControlType.Deny Then
                                    resul(1) = True
                                    If acc.IsInherited Then
                                        resul(2) = True
                                    End If
                                End If
                            End If
                        End If
                    End If
                End If

            Next
        Catch e As FormatException
            MsgBox("Error de formato al obtener los permisos sobre " & carpeta & e.Message)
        Catch e As AccessViolationException
            MsgBox("Error al obtener los permisos, No tienes permisos para acceder a este dato " & e.Message)
        Catch e As ArgumentOutOfRangeException
            MsgBox("Error al obtener los permisos,Se ha intentado obtener un dato con un índice fuera de rango en el large integer " & e.Message)
        Catch e As OperationCanceledException
            MsgBox("Error al obtener los permisos,Se está intentando obtener un dato involucrado en una operación cancelada en el large integer " & e.Message)
        Catch e As OperationAbortedException
            MsgBox("Error al obtener los permisos,Se está intentando obtener un dato involucrado en una operación abortada en el large integer " & e.Message)
        Catch e As OutOfMemoryException
            MsgBox("Error al obtener los permisos,La memoria está totalmente ocupada " & e.Message)
        Catch e As FieldAccessException
            MsgBox("Error al obtener los permisos,  acceder a al ubicación física del dato en el large integer " & e.Message)
        Catch e As InsufficientMemoryException
            MsgBox("Error al obtener los permisos,La memoria está totalmente ocupada " & e.Message)
        Catch e As MissingFieldException
            MsgBox("Error al obtener los permisos, al acceder a al ubicación física del dato, no existe " & e.Message)
        End Try

        oImp1.EndImpersonation()
        Return resul
    End Function

End Class
